/*
 * DemoApplet.java
 *
 * Created on 01 February 2005, 14:48
 */

package MiscTests;

/**
 *
 * @author  Jean-Alain Grunchec
 */
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;

/** Common class for demo applets.
 * 
 * Displays a launch button on the web page, and 
 * creates a separate applet window when it is pressed.
 */
public class DemoApplet extends Applet {
    private String name;
    private DemoFrame frame = null;
    
    /** Create or redisplay a frame in response to a button press */
    private void createFrame() {
        if (frame != null) {
            // (Withdraw and redisplay ???why) so that it moves to the front
            //frame.setVisible(false);
            //frame.show();
            // ... perhaps it de-iconified the frame on some platforms.
            frame.toFront();
        } 
        else {
            try {
                Class appClass = Class.forName(name);
                Constructor cons = 
                    appClass.getConstructor(new Class[] { Boolean.TYPE });
                frame = (DemoFrame) cons.newInstance(new Object[] { Boolean.TRUE });
                frame.initApplet(this);         
                frame.addWindowListener(new WindowAdapter() {
                    public void WindowOpened(WindowEvent e) { frame.toFront(); }
                });
                frame.setVisible(true);
            }
            catch (Exception e) { System.out.println(e); }
        } 
    }
    
    public DemoFrame getFrame() { 
        if (frame == null) createFrame();
        return frame; 
    }
    
    public void forgetFrame() {
        frame = null;
    }
    
    /** Get color parameter or use specified default */
    private Color getColor(String name, Color deflt) {
        String spec = getParameter(name);
        if (spec == null) return deflt;
        return new Color(Integer.parseInt(spec, 16));
    }
    
    public void init() {
        setLayout(new GridBagLayout()); 
        name = getParameter("name");
        setBackground(getColor("background", getBackground()));
        Button launch = new Button("Launch " + name);
        launch.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { createFrame(); }
        });
        add(launch);
    }
    
    public void stop() {
        if (frame != null) frame.die();
    }   
}
