/*
 * EditArrayPanel.java
 *
 * Created on 09 July 2004, 15:21
 */

package guikit;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.TableModel;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import neuroml.core.*;
import neuroml.util.*;
import neuroml.sim.*;
/**
 *
 * @author  Jean-Alain Grunchec
 */
public class EditArrayPanel extends EditPanel implements TableModelListener {
  /** The set / list
     */


    /** If it's a set...
     */
    neuroml.core.NeuromlArray theset;

    /** The actual elements of the set/list
     */

    /** Base class for new objects
     */
    protected Object baseClass; 

    /** Namespace
     */
    protected Namespace ns;

    private javax.swing.JLabel jLabel_Column;
    private javax.swing.JLabel jLabel_Line;
    private javax.swing.JPanel jPanel_Top;
    private javax.swing.JPanel jPanel_Line;
    private javax.swing.JPanel jPanel_Column;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSpinner jSpinner_Column;
    private javax.swing.JSpinner jSpinner_Line;
    private javax.swing.JTable jTable;
    private Integer ColumnNumber = new Integer(0);
    private Integer LineNumber = new Integer(0);
    
    private Object[][] ContentArray;
    private String[] TitleArray;
    private Class[] TypesArray;
    private boolean[] canEdit;
    
    /** Creates a new instance of EditArrayPanel */
    public EditArrayPanel(ObjectNode node) { 	
	super(node);
    }

    /** Build the set panel
     */
    public EditArrayPanel(Namespace ns, ObjectNode node) {
	super(node);
	//System.out.println("EditArrayPanel panel path="+node.getPath());
	this.ns = ns;    
	theset = (neuroml.core.NeuromlArray)(node.getContent());
	baseClass = theset.getDefaultValue();
        ColumnNumber=new Integer(theset.NbColumns);
        LineNumber=new Integer(theset.NbLines);
        
	makePanel();
    }
    private void makePanel() { 
        jPanel_Top = new javax.swing.JPanel();
        jPanel_Column = new javax.swing.JPanel();
        jLabel_Column = new javax.swing.JLabel();
        jSpinner_Column = new javax.swing.JSpinner();
        jPanel_Line = new javax.swing.JPanel();
        jLabel_Line = new javax.swing.JLabel();
        jSpinner_Line = new javax.swing.JSpinner();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTable = new javax.swing.JTable();
        
        jSpinner_Column.setModel(new SpinnerNumberModel(ColumnNumber,new Integer(0),new Integer(65535),new Integer(1)));
        jSpinner_Line.setModel(new SpinnerNumberModel(LineNumber,new Integer(0),new Integer(65535),new Integer(1)));
        //jSpinner_Column.getModel().addChangeListener(this);
        jSpinner_Column.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                ColumnNumber=new Integer(((SpinnerNumberModel)(jSpinner_Column.getModel())).getNumber().intValue());
                theset.setNbColumn(ColumnNumber.intValue());
                RefreshTable();
                //change the NeuromlArray
            }
        });
        jSpinner_Line.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                LineNumber=new Integer(((SpinnerNumberModel)(jSpinner_Line.getModel())).getNumber().intValue());
                theset.setNbLines(LineNumber.intValue());
                RefreshTable();
            }
        });
        
        jPanel_Top.setBorder(new javax.swing.border.TitledBorder("Dimensions of the array"));
        jLabel_Column.setText("Columns");
        jPanel_Column.add(jLabel_Column);
        jPanel_Column.add(jSpinner_Column);
        if(theset.getEditNbColumns())
            jPanel_Top.add(jPanel_Column);
        jLabel_Line.setText("Lines");
        jPanel_Line.add(jLabel_Line);
        jPanel_Line.add(jSpinner_Line);
        if(theset.getEditNbLines())
            jPanel_Top.add(jPanel_Line);
        setLayout(new BorderLayout());
        add(jPanel_Top, java.awt.BorderLayout.NORTH);
        jScrollPane1.add(jTable);
        add(jScrollPane1, java.awt.BorderLayout.CENTER);
        RefreshTable();
    }
    
    private void RefreshTable(){
        jScrollPane1.remove(jTable);
        jTable = new javax.swing.JTable();
        
        TitleArray = new String[theset.NbColumns+1];
        TitleArray[0]=new String("");
        TypesArray=new Class[theset.NbColumns+1];
        TypesArray[0]=String.class;
        canEdit=new boolean[theset.NbColumns+1];
        canEdit[0]=false;
        ContentArray=new Object[theset.NbLines][theset.NbColumns+1];
        for(int i=0;i<theset.NbColumns;i++){
            TitleArray[i+1]=theset.getTitleColumn(i);
            TypesArray[i+1]=theset.getDefaultValue().getClass();
            canEdit[i+1]=true;
            for(int j=0;j<theset.NbLines;j++){
                ContentArray[j][i+1]=theset.getValue(j,i);
            }
        }
        for(int i=0;i<theset.NbLines;i++){
            //ContentArray[0][i]=theset.getTitleLines(i);
            ContentArray[i][0]=theset.getTitleLines(i);
        }
        jTable.setModel(new javax.swing.table.DefaultTableModel(
            ContentArray,
            TitleArray
        ) {
            public Class getColumnClass(int columnIndex) {
                return TypesArray [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
        jTable.getModel().addTableModelListener(this);
        jScrollPane1.setViewportView(jTable);  
    }
    public void tableChanged(TableModelEvent e) {
        int row = e.getFirstRow();
        int column = e.getColumn();
        TableModel model = (TableModel)(e.getSource());
        String columnName = model.getColumnName(column);
        Object data = model.getValueAt(row, column);
        theset.setValue(row,column-1,data);
    }
    
    public static void main(String[] args){
        NeuromlArray o=new NeuromlArray();
        o.setDefaultValue(new Double(1.0));
        o.setNbColumn(1);
        o.setNbLines(1);
        ObjectNode node=new ObjectNode(o);
        JFrame stuff = new JFrame("Test");
        stuff.getContentPane().setLayout(new BorderLayout());
        JPanel muche = new EditArrayPanel(new Namespace("myName"),node);
        stuff.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
               System.exit(0);
            }
        });
        stuff.getContentPane().add(muche);
        stuff.pack();
        stuff.setVisible(true);
    }
    
    /*
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
     */
}

