package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import neuroml.core.*;
import neuroml.util.*;


/** Double field + units editor 
 */
public class EditDValuePanel extends EditPanel implements ActionListener, DocumentListener {

    Namespace ns;
    DValue val;
    double min, max;
    DecimalField paramValueField;
    JSlider slider;

    /** Build the panel
     */
    public EditDValuePanel(Namespace ns, ObjectNode node) {
	super(node);
 
	val = (DValue)node.getContent();

	slider = makeSlider(node.getName(),val);
	this.ns = ns;
    }
    boolean busy = false; // flag for mutex

   
    /**
     * Display a slider for 
     *
     */
    public JSlider makeSlider(String title, DValue value ) {
	JLabel paramLabel = new JLabel(title);
	Unit unit = Unit.getStandardUnit(value.unit.getTargetName());
	min = unit.getMinValue();
	max = unit.getMaxValue();
	paramLabel.setPreferredSize( new Dimension(80, 32));
	paramLabel.setMinimumSize( new Dimension(10, 32));// For resizing
	paramValueField = new DecimalField(value.getValue(),10); // For typing in value
	paramValueField.getDocument().addDocumentListener( this );
        paramValueField.setPreferredSize( new Dimension(40, 32));
	paramValueField.setMinimumSize( new Dimension(10, 32));

        slider = new JSlider((int)min,(int)max,(int)value.getValue());
        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
		if (!busy) paramValueField.setValue(slider.getValue());
            }
	    });
	slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
        slider.setPreferredSize( new Dimension(100, 32));
	slider.setMinimumSize( new Dimension(10, 32));
 	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gb);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.weightx = 5.0;
 
	add(paramLabel);      gb.setConstraints(paramLabel,c);
	c.weightx = 10.0;
	c.gridx = 1;
	add(slider);          gb.setConstraints(slider,c);
 
	c.weightx = 5.0;
	c.gridx = 2;
 	add(paramValueField); gb.setConstraints(paramValueField,c);
	// List of possible targets...
        String unitFamilyName = Unit.getUnitFamily(value.unit.getTargetName());
        Hashtable unitList = Unit.getStandardUnits(unitFamilyName);
	// Make a combo box
        JComboBox comboBox = makeComboBox(unitList, val.getUnitRef().getTargetName(), this);
	c.gridx = 3;
	add(comboBox);          gb.setConstraints(comboBox,c);
	return slider;
    }      
 
    // Listen to text field changes
    public void insertUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }

    public void removeUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }

    public void changedUpdate(DocumentEvent e) {
    }

    public void updateDisplayValue(DocumentEvent e) {
	busy = true;
	val.setValue(paramValueField.getValue());
	slider.setValue((int)val.getValue());	
	confirm();
	busy = false;
    }

    /** Update display from values...
     */
    public void updateDisplay(Object newob) {
	paramValueField.setValue( val.getValue() );
	updateDisplayValue(null);
    }

    /** Listen to buttons
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
	JComboBox cb = (JComboBox)e.getSource();
	String selectName = (String)cb.getSelectedItem();
	//System.out.println("You chose "+selectName);
    }

    public void confirm() {
	node.setContent(val);
    }
}
