package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import neuroml.sim.*;
/** Double field editor
 */
public class EditDoublePanel extends EditPanel implements DocumentListener {
    double value;       // Value
    JSlider slider = null;
    double smin, smax;
    JLabel paramLabel;
    DecimalField paramValueField;

    ElementChangedListener notifyParent = null;

    double log10(double d) {
	return Math.log(d)/Math.log(10);
    }

  /** Build the panel
     */
    public EditDoublePanel(ObjectNode node) {
	this(node,node.getName());
    }
    /** Build the panel
     */
    public EditDoublePanel(ObjectNode node, String label) {
	super(node);
	smin = 0;
	Double val = (Double)node.getContent();
	value = val.doubleValue();
	rescale(value);
 
	//System.out.println("Making a Double slider range "+smin+" "+smax);
	setMinimumSize(new Dimension(200,100));
	slider = makeSlider(label,value,smin,smax);
    }
    
    /** Add notification of parent */
    public void addElementChangedListener( ElementChangedListener p ) {
	notifyParent = p;
    }


    void rescale( double v ) {
	DisplayHints h = node.getDisplayHints();
	if (h!=null) {	    
	    smin = h.getMin();
	    smax = h.getMax();
	    //System.out.println("EditDoublePanel set range to "+smin+" "+smax);
	    if ((v > smax) && h.isRangeFixed()) {
		System.out.println("Error - range out of bounds "+v);
		if (v<0.00001) { v = 1.0; }
		double lg = log10(v);	
		smax = Math.pow(10,Math.floor(lg-0.01)+1);		   
	    }	    
	} else {
	    if (v<0.00001) { v = 1.0; }
	    double lg = log10(v);	
	    smax = Math.pow(10,Math.floor(lg-0.01)+1);	
	}
    }

    boolean busy = false; // flag for mutex

    public void setMax( double max ) {
	smax = max;
	updateDisplayValue(null);
    }
    public int valToSlider( double v ) {
	int sv = (int)(100.0 * (v-smin)/smax);
	if (sv < 0) sv = 0;
	if (sv > 100) sv = 100; 
	return sv;
    }
    public double sliderToVal( int v ) {
	return (double)smin + v * (smax-smin) / 100.0;
    }
    
    /** Sets min/man labels on display */
    void setMinMaxDisplay(double min, double max) {
	if (slider!=null) {
	    Hashtable labelTable = new Hashtable();
	    JLabel l1 = new JLabel(""+min); l1.setFont(GuiKit.getDefaultFont());
	    JLabel l2 = new JLabel(""+max); l2.setFont(GuiKit.getDefaultFont());
	    labelTable.put( new Integer( 0 ), l1 );
	    labelTable.put( new Integer( 100 ), l2);
	    slider.setLabelTable( labelTable );	
	}
    }

    /**
     * Display a slider for int number parameter
     *
     */
    public JSlider makeSlider(String title, double value, double min, double max) {
	
 
	paramLabel = new JLabel(title);
	paramLabel.setPreferredSize( new Dimension(80, 32));
	paramLabel.setMinimumSize( new Dimension(10, 32));// For resizing
	paramLabel.setFont( GuiKit.getDefaultFont() );
	paramValueField = new DecimalField((double)value,10); // For typing in value
	paramValueField.getDocument().addDocumentListener( this );
	//	paramValueField.setPreferredSize( new Dimension(-1, 16));
        paramValueField.setPreferredSize(new Dimension(40,32));
        paramValueField.setMinimumSize( new Dimension(10, 32));
        
        slider = new JSlider(0,100,valToSlider(value));
	slider.setMajorTickSpacing(10);
        slider.setPreferredSize( new Dimension(100, 32));
	slider.setMinimumSize( new Dimension(10, 32));// For resizing
	// slider.setMinorTickSpacing(1);
	// slider.setPaintTicks(true);
        slider.setPaintLabels(true);
	rescale(value);
	setMinMaxDisplay(min,max);

        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
		if (!busy) paramValueField.setValue(sliderToVal(slider.getValue()));
            }
	    });
	//	slider.setPreferredSize(new Dimension(-1,20));
	slider.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
 	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gb);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.weightx = 5.0;
	//	c.gridwidth = 10;
	add(paramLabel);      gb.setConstraints(paramLabel,c);
	c.weightx = 10.0;
	c.gridx = 1;
	add(slider);          gb.setConstraints(slider,c);
	//	c.gridwidth = GridBagConstraints.REMAINDER;
	c.weightx = 5.0;
	c.gridx = 2;
 	add(paramValueField); gb.setConstraints(paramValueField,c);
	return slider;
    }      

    // Listen to text field changes
    public void insertUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }

    public void removeUpdate(DocumentEvent e) {
	updateDisplayValue(e);
    }

    public void changedUpdate(DocumentEvent e) {
    }

    public void updateDisplayValue(DocumentEvent e) {
	busy = true;
	value = paramValueField.getValue();
	if (value > smax) {
	    rescale(value);
	    setMinMaxDisplay(smin,smax);
	}
	slider.setValue(valToSlider(value));	
	
	/** Reset the max/min values... */
	

	confirm();
	busy = false;
    }

    /** Update display from values...
     */
    public void updateDisplay(Object newob) {
	//System.out.println("updating display to "+newob);
	try {
	    value = ((Double)newob).doubleValue();
	    paramValueField.setValue( value );
	    updateDisplayValue(null);
	} catch (Exception e) {}
    }

    /** Update real object 
     */
    public void confirm() {
	node.setContent(new Double( value ) );
	/** tell parent that things have changed... */
	if (notifyParent!=null) {	    
	    notifyParent.elementChanged( node,node );
	}
    }  
}
