package guikit;

import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.NumberFormat;

import neuroml.core.*;
import neuroml.util.*;
import neuroml.sim.*;

/** Edit a reference
 */
public class EditRefPanel extends EditPanel implements ActionListener {
    Ref reference;
    Namespace ns;
    JComboBox comboBox=null;

    boolean debug = false;

    ElementChangedListener notifyParent = null;

    /** Add notification of parent */
    public void addElementChangedListener( ElementChangedListener p ) {
	notifyParent = p;
    }

    /** Build the panel
     */
    public EditRefPanel(Namespace ns, ObjectNode node) {
	super(node);
	this.ns = ns;

	JLabel paramLabel = new JLabel(node.getName());
	paramLabel.setPreferredSize( new Dimension(100, 32));
	paramLabel.setMinimumSize( new Dimension(10, 32));// For resizing

	

	GridBagLayout gb = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	setLayout(gb);
        c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.weightx = 5.0;
	add(paramLabel);  
        gb.setConstraints(paramLabel,c);
	refresh();

	// Make a "Edit" button
	JButton ed = makeButton("Edit","Edit",this);
	c.weightx=5.0;
	c.gridx=2;
	add(ed); gb.setConstraints(ed,c);
    }      

    /** Update real object 
     */
    public void confirm() {
	node.setContent(reference);
    }  
    
    public void refresh(){
        if(comboBox!=null){
            Container parent=comboBox.getParent();
            if(parent!=null)
                parent.remove(comboBox);
        }
        
        reference = (Ref)node.getContent();
	String tname = reference.getTargetName();
	String type  = reference.getTargetType();           
        
        GridBagConstraints c = new GridBagConstraints();
        GridBagLayout gb=(GridBagLayout)getLayout();
        c.fill = GridBagConstraints.HORIZONTAL;

	c.weightx = 10.0;
	c.gridx = 1;

	// Make a combo box
        //System.out.println("ns.getReferences() type="+type+" can be:");
        
	Hashtable options = (Hashtable)ns.getReferences().get(type);
        for(Enumeration enum_=options.elements();enum_.hasMoreElements();)
        {
            Object obj_=enum_.nextElement();
           // System.out.println("\t"+obj_.toString());
        }
        //System.out.println("");
        
	if (debug) {
	    System.out.println("Available options for "+type+" in "+ns.getName()+" : "+options+" ns "+ns+" tname "+tname);
	}
        comboBox = makeComboBox(options,  tname, this );
	add(comboBox);          gb.setConstraints(comboBox,c);
    }
       
    /** Listen to buttons
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
	if (e.getActionCommand().equals("Edit")) {
	    Vector lis = ns.getListInfo( reference.getTargetType() );
	    if (lis!=null) {
		// Pop up all appropriate lists..
		// Better to find the nearest to this one..
		// But then we need to know where we are, "path" or sth
		for (int i=0; i<lis.size(); i++) {
		    ListInfo li = (ListInfo)lis.elementAt(i);
		    System.out.println(li.getPath()+" "+ li.getFieldName());
		    if (i==0) {
			EditObjectPanel eop = new EditObjectPanel( ns, li.getOwner(), reference.getTargetType(), false );
			eop.popup();
			// some way to refresh the list here...
		    }
		}
	    }
	} else {
	    JComboBox cb = (JComboBox)e.getSource();
            String selectName=null;
            if(cb.getSelectedIndex()>0){
                selectName = (String)cb.getSelectedItem();
                //ns.deRef(reference,selectName);//Added for the name reference problem
            }
                
	    reference.setTargetName(selectName);
	    System.out.println("You chose "+selectName);
	    if (notifyParent != null) {
		notifyParent.elementChanged( node.getContent(), reference );
	    }
	}
    }
}
