package guikit;

import java.io.*;
import java.io.IOException;
import javax.swing.*; 
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.util.*;


import neuroml.*;
import neuroml.util.*;
import neuroml.core.*;
import neuroml.model.network.*;
 

/** An menu-free Basic NeuroML Applet window with a File menu
 * Intended to be used as the superclass for NeuroML applications.
 *
 * @author  Fred Howell
 * @date July 2001
 */
public class EmptyApp extends JFrame implements ActionListener {

    /** Environment for editors
     */
    public Environment env;

    /** The top level object being edited 
     */
    public Object topObject = null;
    
    /** The namespace for the edited model
     */
    public Namespace ns = null;

    /** Current filename
     */
    public String filename="test.xml";
    
    /** The menu bar
     */
    public JMenuBar menuBar;    
    public JMenu fileMenu;
    public JMenu editMenu;

    /** The object browser dialog
     */
    BrowseObjectsPanel objBrowser;

    /** The modules browser dialog
     */
    BrowseModulesPanel modBrowser;

    /** The about dialog
     */
    AboutPanel aboutPanel;

    /** Resources used for images
     */
    private static ResourceBundle resources;
    static {
        try {
            resources = ResourceBundle.getBundle("guikit.resources/BasicApp",  Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("guikit.resources/BasicApp.properties not found");
        }
    }
    ImageUtils imgLoader;


    /** Add a menu item to a menu 
     * e.g. addMenu(menuBar,"Test", "This is a test", KeyEvent.VK_A)
     */
    public JMenu addMenu( JMenuBar parent, String name, String desc, int m) {
	JMenu menu = new JMenu(name);
	if (m!=-1)  menu.setMnemonic(m);
        menu.getAccessibleContext().setAccessibleDescription(desc); 
	parent.add(menu);
	menu.addActionListener(this);
	return menu;
    }

    /** Add a menu item to a menu 
     * e.g. addMenu(menuBar,"Test", "This is a test", KeyEvent.VK_A)
     */
    public JMenu addMenu( JMenu parent, String name, String desc, int m) {
	JMenu menu = new JMenu(name);
	if (m!=-1)  menu.setMnemonic(m);
        menu.getAccessibleContext().setAccessibleDescription(desc); 
	parent.add(menu);
	menu.addActionListener(this);
	return menu;
    }

    public JMenuItem addMenuItem( JMenu parent, String name, String desc, int m) {
	JMenuItem mi = new JMenuItem(name);
	if (m!=-1) 
	    mi.setAccelerator(KeyStroke.getKeyStroke(m, ActionEvent.ALT_MASK)); 
        mi.getAccessibleContext().setAccessibleDescription(desc); 
	mi.addActionListener(this);
	parent.add(mi);
	return mi;
    }

    /** Create the main menu
     * Override this for customised menu
     */
    public void createMenu() {
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);


	//menuBar.add(Box.createHorizontalGlue());


    }     
    
    
    /** Popup a file chooser 
     */
    public String fileChooser() {
	String cwd = System.getProperty("user.dir");
	JFileChooser chooser = new JFileChooser(cwd);  
	ExampleFileFilter filter = new ExampleFileFilter( "xml", "NeuroML model files");
	chooser.addChoosableFileFilter(filter);	
	int returnVal = chooser.showOpenDialog(this); 
	if (returnVal == JFileChooser.APPROVE_OPTION) { 
	    return chooser.getSelectedFile().getName();
	} 
	return null;
    }
    
    /** Top level edit object method - 
     * Replace namespace with new object.
     * Derived versions can do as they please.
     */
    public void editObject( Namespace ns, Object o ) {
	topObject = o;
	this.ns = ns;
	EditObjectPanel eop = new EditObjectPanel( ns,
						   topObject, 
						   o.getClass().getName(),
						   false );
	eop.popup();
    }

    /** Add the object to existing model
     */
    public void importObject( Namespace ns, Object o ) {
	System.out.println("Import object not yet done - ");
	// *** Need to merge namespaces ***
    }
    
    /** Respond to menu choices
     */
    public void actionPerformed(ActionEvent e) {
	update(getGraphics());
	System.out.println("Unknown command");
       
    }
    
    /** Initialise application
     */
    public EmptyApp(String frameName) {
        super(frameName);   
       	
	imgLoader = new ImageUtils(this,resources);

	SpecialEditors.put("guikit.RGBColour", "guikit.ColourEditor" );
	SpecialEditors.put("neuroml.model.network.Network", "guikit.SubnetEditor" );

	ns = NeuromlSys.makeNamespace("editor"); 
	NeuromlPkg np = new NeuromlPkg();
	np.setName("New Model");
	topObject = np;
	NeuromlSys.addReferences(np, "editor");
	createMenu();

	// Set environment for sub editors.
	env = new Environment( ns, np, imgLoader );
	SpecialEditors.setEnvironment(env);

	JPanel tmp = new JPanel();
	tmp.setBackground(Color.gray);
	tmp.setLayout( new BorderLayout(5,5) ); 
	getContentPane().add(tmp);
	tmp.setPreferredSize(new Dimension(300,100));
    }
   
    /** Sample main program
     */
    public static void main(String[] args) {
	System.out.println("Starting Empty Applet");
        JFrame frame = new BasicApp("Empty neuroml editor applet");
	frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) { System.exit(0);  }
	    });  	
        frame.pack();
        frame.setVisible(true);
    } 
}
