package neosim.iface;
import java.util.Vector;
import java.util.Hashtable;

/** An EntityClass contains all information which is
 * common between different instantiations of an entity,
 * and a <b>makeEntity</b> method to build entities.
 * @author Fred Howell
 */
public interface EntityClass extends NeosimClass {
    /** An EntityClass must provide a method to
     * create an instance of the class
     * @param p the population the entity will be part of
     * @param index the index in the population
     */
    public Entity makeEntity(Population p, int index);

    /** Returns the name of this class 
     * @returns the name of this class 
     */ 
    public String getName();

    /** Initialise from a bean - this is a temp placeholder 
     * @deprecated
     */
    public void init( String name, String args );

    /** Initialise from a bean - with a hashtable of arguments 
     * @param name the name of the entity class
     * @param args string name/value pairs for arguments
     */
    public void init( String name, Hashtable args );

    /** Add an output port to the class. This port will 
     * be valid for all entity instances of the class.
     * @param ec the class of events the port generates
     * @param name the name of the port (optional)
     * @return the id of the output port (from 1..#output ports).
     */
    short  addOutPort(EventClass ec, String name);//PortID
    /** Add an output port to the class. This port will 
     * be valid for all entity instances of the class.
     * @param ec the class of events the port generates   
     * @return the id of the output port (from 1..#output ports).
     */
     short addOutPort(EventClass ec);//PortID

    /** Add an input port to the class. This port will 
     * be valid for all entity instances of the class.
     * @param ec the class of events the port accepts
     * @param name the name of the port (optional)
     * @return the id of the input port (from 1..#input ports).
     */
     short addInPort (EventClass ec, String name);//PortID
    /** Add an input port to the class. This port will 
     * be valid for all entity instances of the class.
     * @param ec the class of events the port accepts
     * @return the id of the input port (from 1..#input ports).
     */
     short addInPort (EventClass ec);//PortID

    /** Return the vector of output ports
     * @return  the vector of output ports
     */
    Vector getOutPorts();
    
    /** Return the vector of input ports
     * @return  the vector of input ports
     */
    Vector getInPorts();
    
    /** Get an output port given a name
     * @param name the name of the port
     * @return the id of the port
     */
    short getOutPort( String name );//PortID

    /** Get an input port given a name.
     * By default ports are named "in1, in2, ..." and "out1, out2, ..."
     * @param name the name of the port
     * @return the id of the port
     */
    short getInPort( String name );//PortID

    /** If this entity requires extra 
     * global initialisation (e.g. if it requires 
     * installing a C++ libxxx.so file on all processors)
     * this method can be overridden to initialise the module.
     * It is likely that this mechanism will be replaced
     * by something more elegant.
     * @param sr the calling script reader
     */
    void initModule( ScriptReader sr );
    
    /**Ensure that the EntityClass is Constructed in the asking Kernel.
     */
    boolean IsInParent();
}

