package neosim.iface;

/** Top level interface for 
 * registering neosim classes for C++ object serialization.
 * @author Fred Howell
 */
public interface NeosimClasses {
    /** Return a class from an integer id
     * @param cid the class id
     * @return the NeosimClass, or null if an invalid id
     */
    public NeosimClass getClassFromID( ClassID cid );

    /** Return a class from a name
     * @param name the name of the class
     * @return the NeosimClass, or null if not found
     */
    public NeosimClass getClassFromName( String name );

    /** Return a class id from a name
     * @param name the name of the class
     * @return the ClassID, or -1 if an invalid name
     */
    public ClassID     getClassIDFromName( String name );

    /** Add a class to the registry
     * @param nc the class to add
     * @return the id of the class
     */
    public ClassID     addClass(NeosimClass nc);

    /** Register a class given name, parent, and package.
     * e.g. <tt>registerClass( "SpikeEvent", "Event", "neosim.kernel" )</tt>
     * @param name the name of the class
     * @param parent the name of the parent
     * @pkg the package name
     */
    public ClassID     registerClass(String name, String parent, String pkg);

    /** Convert an instance into an array of bytes (object serialization)
     * @param o the object to convert
     * @return the array of bytes 
     */
    public byte[]      getByteArray(Object o);
    
    /** Read an object from an array of bytes
     * @param a the array
     * @return the object
     */
    public Object      readFromArray(byte[] a);
}
