package neosim.implementation;

import neosim.iface.*;
import neosim.kernel.*;
import java.util.Hashtable;

/** The part of a general projection which runs on
 * a destination entity and decides to fulfil/deny
 * connection requests.
 */
public class DestMethodImpl implements DestMethod {
    protected short destPort;//PortID
    protected Time delay;
 //   transient neosim.kernel.DeadlockTracker tracker=new neosim.kernel.DeadlockTracker();

    public DestMethodImpl() { 	
    }

    public DestMethodImpl(short destPort, Time delay ) { //PortID
	this.destPort = destPort; 
	this.delay    = delay;
    }

    public final short getDestPort() { return destPort; }//PortID
    public final Time getDelay() { return delay; }

    /** The considerRequest method checks the source entity's details 
     * against the dest entity's details, and returns True or False
     * depending on whether or not the connection will be made. 
     * @param deste the destination entity
     * @param cr the connection request from the source entity
     * @return connection if a connection made, null otherwise.
     */
    public Connection considerRequest( Entity deste, ConnectionRequest cr , boolean BuildingConnection) {
	// *** Here, always making a connection. ***
	// *** But in general, this method should be overridden
	Connection c = new ConnectionSpec( cr.getSrcEntityID(), cr.getSrcPortID(), 
					   deste.getID(), destPort, delay.t );

	makeConnection(deste, c,BuildingConnection );
	return c;
	// else return null
    }
 
    /** 
     * This method is called by the considerRequest method to 
     * create a connection.
     * @param c the connection to make
     * @return the id of the connection
     */
    public final int makeConnection( Entity deste, Connection c, boolean BuildingConnection ) {	//ConnectionID
	ConnectionImpl ci = new ConnectionImpl( c.getSrcEntityID(), 
						c.getSrcPortID() ,
						(EntityImpl)deste, 
						c.getDstPortID() , 
						c.getDelay() );
	
	// This method of adding connections seems clumsy...
	BasicKernel bk    = (BasicKernel) SimSystem.getKernel();
			       
	int srcproc  = bk.getEntityProcessor(c.getSrcEntityID());
        
        //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ci,ecp);         
	int cid=bk.addInConnection( srcproc, ci, BuildingConnection ); 
        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
	// c.setDstParams(deste, ci.getConnectionID());	

	return cid;//new ConnectionID(
    }

    /** 
     * This method is called when the method is loaded 
     * from a jar file
     * 
     * @param name name of the method
     * @param params hashtable of the method's parameters
     *
     */
    public void init( String name, Hashtable params) {
    }
}
