

package neosim.implementation;
import neosim.iface.*;
import neuroml.sim.state.PositionState;
import neuroml.util.module.*;
import neosim.neuroml.*;
import neuroml.sim.run.*;
import neuroml.model.I_Clamp.*;


/** 
 * A Population builder which constructs a 3D mesh
 * of cells with a given start point / spacing.
 * Requires entities with a position.
 */
public class Grid3DPopulation extends PopulationBuilderImpl { 

    int    xdim, ydim, zdim; 
    double xorig, yorig, zorig;
    double xdelta, ydelta, zdelta;
    double xrand, yrand, zrand; 
    
    /** Builds size instances of entity class ec.
     * Entities must implement the Entity3D.
     * @param name the population name
     * @param parent the parent population
     * @param ec the class of entities to build
     * @param ei the initialisation to apply to each entity
     */
        public Grid3DPopulation( String name, 
			     //PopulationNode parent, //SOCKET_ERROR
			     int xdim, int ydim, int zdim, 
			     double xorig, double yorig, double zorig,
			     double xdelta, double ydelta, double zdelta,
			     EntityClass ec, I_Clamp stuff ) { 

	super(name, xdim*ydim*zdim,ec, new EntityInit() { //SOCKET_ERROR , parent
		public final void init( Entity e, Population p, int index) {
		    // Make sure Entity has a setPos method
		    // Entity3D could be an interface? - better.
                   
		    try {
                        PositionState e3d = (PositionState) e;
                        Grid3DPopulation p3d = (Grid3DPopulation) p; 
			p3d.setEntityPos( e3d, index );
                        if(e instanceof NeosimEntityImpl){
                            RunnableCell rc=((NeosimEntityImpl)e).getRunnableCell();
                            if(rc instanceof BasicRunnableCell){
                                _Function stuff;
                                if(p3d.Intensity==null){
                                    stuff=new _Function_step(0.0);
                                }
                                else{
                                    stuff=p3d.getFunction(index);
                                }
                                ((BasicRunnableCell)rc).setIntensity(stuff);                                
                            }
                        }
		    } catch (Exception ex) {
			System.out.println("Error - Grid3DPopulation can only be made for Entity3D \t"+ex);
                        ex.printStackTrace();
		    }
		}
	    }	
	      , stuff);
	this.xdim      = xdim;          this.ydim      = ydim;          this.zdim      = zdim;
        this.xorig     = xorig;         this.yorig     = yorig;         this.zorig     = zorig;
        this.xdelta    = xdelta;        this.ydelta    = ydelta;        this.zdelta    = zdelta;
        this.xrand     = 0;             this.yrand     = 0;             this.zrand     = 0;	
    }

    /** Get x index given global idx */
    public final int getXidx( int globi ) {   return globi % xdim;   }
    public final int getYidx( int globi ) {   return (globi / xdim) % ydim;  }
    public final int getZidx( int globi ) {   return (globi / (xdim*ydim));  }

    /** Get global idx given x,y,z */
    public final int getIndex( int x, int y, int z ) {   
	return x + y*xdim + z*xdim*ydim;   
    }
    
    /** Set entity pos */
    public final void setEntityPos( PositionState e, int index ) {
	double xp = xorig + getXidx(index)*xdelta; // No random displacement yet
	double yp = yorig + getYidx(index)*ydelta; // No random displacement yet
	double zp = zorig + getZidx(index)*zdelta; // No random displacement yet
	e.setPos( xp,yp,zp );
    }

    /** Return the X dimensions of the grid */
    public final int getXdim() { return xdim; }
    /** Return the Y dimensions of the grid */
    public final int getYdim() { return ydim; }
    /** Return the Z dimensions of the grid */
    public final int getZdim() { return zdim; }

    /** Set the mag of the random displs to add to each position 
     * At present this is a uniform random # +/- these values
     */
    public final void setRandDispls( double x, double y, double z ) {
        this.xrand = x; this.yrand = y; this.zrand = z;
    }

}
