package neosim.implementation;
import neosim.iface.*;
import java.util.Vector;

/** A basic implementation of a register of 
 * classes.  Used for C++ object serialization.
 */
public class NeosimClassesImpl implements NeosimClasses {
    
    Vector classes;

    public NeosimClassesImpl() {
	classes = new Vector();
    }

    /** Return a class from an integer id
     * @param cid the class id
     * @return the NeosimClass, or null if an invalid id
     */
    public final NeosimClass getClassFromID( ClassID cid ) {
	return (NeosimClass)classes.elementAt( cid.id );
    }

    /** Return a class from a name
     * @param name the name of the class
     * @return the NeosimClass, or null if not found
     */
    public final NeosimClass getClassFromName( String name ) {
	NeosimClass ret = null;
	for (int i=0; i<classes.size(); i++) {
	    NeosimClass c = (NeosimClass)classes.elementAt(i);
	    if (c.getName().equals(name)) { ret = c; break; }
	}
	return ret;
    }

    /** Return a class id from a name
     * @param name the name of the class
     * @return the ClassID, or -1 if an invalid name
     */
    public final ClassID     getClassIDFromName( String name ) {
	NeosimClass ret = null;
	for (int i=0; i<classes.size(); i++) {
	    NeosimClass c = (NeosimClass)classes.elementAt(i);
	    if (c.getName().equals(name)) { ret = c; break; }
	}
	return ret.getClassID();
    }

    /** Add a class to the registry
     * @param nc the class to add
     * @return the id of the class
     */
    public final ClassID     addClass(NeosimClass nc) {
	classes.addElement( nc ); 
	return nc.getClassID();
    }

    /** Register a class given name, parent, and package.
     * e.g. <tt>registerClass( "SpikeEvent", "Event", "neosim.kernel" )</tt>
     * @param name the name of the class
     * @param parent the name of the parent
     * @pkg the package name
     */
    public final ClassID     registerClass(String name, String parent, String pkg) {
	ClassID nc = new ClassID(classes.size());
	NeosimClass pc = getClassFromName(parent);
	NeosimClassImpl nci = new NeosimClassImpl( pc, name, pkg, nc );
	return addClass( nci );
    }

    /** Convert an instance into an array of bytes (object serialization)
     * @param o the object to convert
     * @return the array of bytes 
     */
    public final byte[]      getByteArray(Object o) {
	return new byte[3]; // *** FIXME ***
    }
    
    /** Read an object from an array of bytes
     * @param a the array
     * @return the object
     */
    public final Object      readFromArray(byte[] a) {
	return new Integer(0); // *** FIXME ***
    }
}
