

package neosim.implementation;
import neosim.iface.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

/**  A population is a collection of one or more entity handles.
 * It can contain extra methods for initialising the elements of
 * the population, or for indexing them rapidly.
 */
public class PopulationImpl extends PopulationNodeImpl implements Population {
    int id;
    int size;
    Vector members;      // vector of EntityID indexed by population index
    Hashtable memberIdx; // vector of indices hashed by entity handle
 
    public PopulationImpl( String name,  int size ) {//SOCKET_ERROR PopulationNode parent,
	super( name );//SOCKET_ERROR , parent
	this.size = size;
	members   = new Vector();
	memberIdx = new Hashtable();
        //Added JAG
        locals = new Vector();
	// get unique population ID from kernel.
    }

    /** Returns unique population id */
    public int getID() {
	return id;
    }

    /** Sets unique population id */
    public void setID(int i) {
	 id = i;
    }

    /** Return number of entities */
    public int getNumEnts() {
	return size;
    }

    /** return index of a given entity handle 
     * @param eid handle of entity
     * @return index of entity in population if present, or -1 otherwise.
     */
    public int getIndex( int eid ) {//EntityID eid
	// lookup eid in members hash table
	Integer idx = (Integer)(memberIdx.get( new Integer(eid) ));
	if (idx==null) return -1; else return idx.intValue();
    }

    /** return index of a given entity handle 
     * @param i index in population
     * @return  handle of entity
     */
    public int getEntityID( int i ) {//EntityID return	
	return ((Integer)(members.elementAt(i))).intValue();//EntityID
    }

    /** return whether a given entity is a member
     * @param eid handle of entity
     * @return true if entity is a member of this population
     */
    public boolean isMember( int eid ) {//EntityID eid
	if (getIndex(eid)==-1) return false; else return true;
    }

    /** Add a member to this population
     * @param eid handle of entity
     */
    public void addMember( int e ) {//EntityID e
	members.addElement( e );
	memberIdx.put( new Integer(e), new Integer(members.size()-1) );
    }


    /** Return a list of all members 
     */
    public Vector getMemberList() {
	return members;
    }

    public boolean isLeaf() { return true; }




    transient Vector locals = new Vector();
    /** Add a local member to this population (kernel method)
     * @param e entity to add
     */
    public void addLocalMember( Entity e ) { 
        if(locals==null)
            locals = new Vector();
        locals.addElement( e ); 
    }

    /** Add a local member to this population (kernel method)
     * @return the vector of lcoal entities
     */
    public Vector getLocalMembers() { return locals; }
}
