package neosim.kernel;

import java.util.*;
import java.lang.reflect.*;

/** The Command Queue runs in a thread and
 * buffers commands (stored as Command Objects).
 * Commands are added using add() and sent on as fast as
 * possible. If output fails, subsequent isdestAlive() 
 * calls return false.
 *
 * @author Fred Howell
 * @date Sept 2000
 * @version 2
 */
public class CommandQueue extends Thread {
    Commandable dest;
   // DeadlockTracker tracker=new DeadlockTracker();
    boolean destAlive = true;
    boolean terminated = false;
    boolean terminated_flushed = false;
    
    List co;
    List delayed_co;
    static private boolean debug_verbose = true;
    static private boolean debug = false;
    
    String name = "not set";
    public void setQName(String s) { name = s; }

    //Semaphore arrived = new Semaphore(0);
    
    int arrived = 0;
    Integer IArrived = new Integer(0);
    
    Semaphore sent    = new Semaphore(0);
    

    int narrived = 0; // number arrived since last flush.

    int totalArrived = 0;
    int totalSent =0;
    static final int TotalBufferSize=1048576;//about 1Mb and then it should flush this parameter might need to be exported (and it is PER COMMANDQUEUE so it might need to be divided/treated differently in case of several command queues around).
    static int MaxBufferSize=TotalBufferSize;
    static int nbQueues[]=new int[1];
    int currentBufferSize=0;
   
    public CommandQueue(Commandable dest) {
        debug_verbose=false;//nmlplay.NEOSIMSettings.mySchedule.DebugOnOff;
	this.dest = dest;
	co = Collections.synchronizedList( new LinkedList() );
        delayed_co = Collections.synchronizedList( new LinkedList() );
        setName(getName()+" "+this.getClass().getName());
    }
    
    /** Terminate the running thread */
    public final void terminate() {
	if (debug) System.out.println(name+" terminate message arrived arr="+totalArrived+" sent="+totalSent+" left "+narrived);
	terminated = true;
	//arrived.sem_signal();
        //Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp);   
        synchronized(IArrived){
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            arrived++;
            IArrived.notify();
        }
    }
    
    /** Throw an exception rather than blocking if output has failed 
     */
    static int current_i=0;
    int Current_level=0;
    
    public final void add(CommandObj o) throws java.rmi.RemoteException {
	if (debug)	System.out.println("Adding an "+o+" item to a CQ");
	if (destAlive==false) {
	    throw new java.rmi.RemoteException("Queue dest died\n");
	} else {
            try{
                boolean doit=false;
                VectorCommand vc=null;
                //Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp); 
                synchronized(IArrived){
                    synchronized(co){
                        synchronized(delayed_co){
                            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
                 
                    if(currentBufferSize>=MaxBufferSize){
                        //neosim.kernel.Deadlock_Message ecp1=new neosim.kernel.Deadlock_Message();
                        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp1); 
                        doit=true;
                
                        int k=0;
                        while(delayed_co.size()!=0){
                            CommandObj c=(CommandObj)delayed_co.remove(0);
                            if(c!=null){
                                co.add( co.size(), c );
                                k++;
                            }
                        }
                       // 
                        totalArrived +=k;

                        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp1);
                        //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
                        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp2);                     
                        int nsent=1;//1?

                        CommandObj cmd1 = (CommandObj)co.remove( 0 );
                        currentBufferSize-=cmd1.getSize();
                       
                        vc = new VectorCommand(cmd1.getID());
                        if (debug) System.out.println(name+"Doing single command (v):" + cmd1.getType());				
                        vc.add(cmd1);
                        Object debugObject=cmd1;
                        Iterator it = co.iterator();
                        debugObject=vc;
                        while (co.size()>0) {
                            CommandObj cmd2 = (CommandObj)co.remove(0);
                            currentBufferSize-=cmd2.getSize();
                            vc.add(cmd2);
                            nsent ++;
                        }
                        nsent-=k;
                        narrived-= nsent;    
                        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);
                        //neosim.kernel.Deadlock_Message ecp3=new neosim.kernel.Deadlock_Message();
                        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp3);                     

                        for(int j=0; j<nsent; j++){  
                            while(arrived==0){
                                try { 
                                    //neosim.kernel.Deadlock_Message ecp_=new neosim.kernel.Deadlock_Message();
                                    //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp_); 
                                    IArrived.wait();
                                    //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp_);
                                }
                                catch(Exception x){
                                    x.printStackTrace();
                                }
                            }
                            arrived--;
                        
                        }
                        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp3);
                    }//if
                }}}   
                //neosim.kernel.Deadlock_Message ecp4=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(dest,ecp4); 
                if(doit){
                    Current_level++;    
                    System.out.println("CommandQueue: Current Depth Level (event building scheme : "+ Current_level);
                    boolean goon=true;
                    int delay=100;
                    while(goon){
                        try{
                             dest.command(vc);
                             goon=false;
                        }
                        catch(Exception X){
                            System.out.println("Error_1 in CommandQueue.Add "+dest.getClass().getName());
                           // this.PrintObject(vc,"CommandQueueAdd_1_Error.txt");
                            try{
                              /*  java.io.PrintWriter trcstream = new java.io.PrintWriter(new java.io.FileOutputStream("CommandQueueAdd_1_Error.txt"));
                                X.printStackTrace(trcstream); 
                                trcstream.close();
                                X.printStackTrace();*/
                                Thread.sleep(delay);
                                delay+=200;
                            }catch(Exception x){x.printStackTrace();}                            
                        }
                    }
                    Current_level--;
                    System.out.println("CommandQueue: Current Depth Level (event building scheme : "+ Current_level+" passed");
                }
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp4);
                //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp2); 
                synchronized(IArrived){
                    synchronized(co){
                        synchronized(delayed_co){  
                            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);
                            //neosim.kernel.Deadlock_Message ecp5=new neosim.kernel.Deadlock_Message();
                            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp5);                        
                    if(o.isDelayed()){//It is likely to be a measurement
                        delayed_co.add( delayed_co.size(), o );
                        currentBufferSize+=o.getSize();
                    }
                    else{
                        co.add( co.size(), o );
                        if (debug) System.out.println(name+": Adding command :" + o);
                        narrived ++;
                        totalArrived ++;
                        currentBufferSize+=o.getSize();                    
                        arrived++;
                        IArrived.notify();
                    }
                    //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp5);                        
                }}}  
            }
            catch(Exception XCP){
                try{
                    System.out.println("Error_2 in CommandQueue.Add "+dest.getClass().getName());
                    this.PrintObject(o,"CommandQueueAdd_2_Error.txt");
                    java.io.PrintWriter trcstream = new java.io.PrintWriter(new java.io.FileOutputStream("CommandQueueAdd_2_Error.txt"));
                    XCP.printStackTrace(trcstream);
                    trcstream.close();
                    Thread.sleep(15000);
                }catch(Exception x){x.printStackTrace();}
            }
        }
    }
    
    public final boolean isdestAlive() { return destAlive; }
    
    /** Number waiting 
     */
    public final int nwaiting() { return co.size(); }
    
    /** Wait for output queues to empty -
     * Note, just waits for items on q at calling time to disappear
     */
    public final void flush(boolean MeasurementToo) {
	int i;	
        int n=-1;
        if(MeasurementToo){
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp); 
            synchronized(IArrived){
                synchronized(co){
                    synchronized(delayed_co){  
                        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
            int k=0;
  /*          synchronized(co) {
                synchronized(delayed_co) {  */  

                    while(delayed_co.size()!=0){
                        CommandObj c=(CommandObj)delayed_co.remove(0);
                        if(c!=null){
                            co.add( co.size(), c );
                            k++;
                        }
                        else
                            delayed_co=Collections.synchronizedList( new LinkedList() );
                    }
                    narrived +=k;
                    totalArrived +=k;
                    while(k-->0){
                        //arrived.sem_signal();  
                        arrived++;
                        IArrived.notify();                        
                    }
                n = narrived; // Wait for all which have arrived to be sent.
                if (debug) 	System.out.println("CQ Flush "+n);
                narrived=0;
                }}}//sync
        
            if (n>0) {
             //   synchronized(sent){      
        //            System.out.println("Synchronizing 1 : "+n+" "+sent.getCount());
                    for (i=0; i<n; i++) {
                        sent.sem_wait(new Integer(n-i));
                    }
         //       }
         //           System.out.println("Synchronizing 2");
   /*             }
            }*/
            }
       }


//	synchronized(co) {
	   // narrived -= n;
	//}
//	if (debug)	System.out.println(name+"CQ Flush "+n+" done");
    }
    
    public final void run() {
	if (debug)	System.out.println("A Command Queue named "+name+" has started");
        Object debugObject=new Object();

        synchronized(nbQueues){
            nbQueues[0]++;
            MaxBufferSize=TotalBufferSize/nbQueues[0];
        }
	while(!terminated_flushed) {
            int nw=-1; 
            CommandObj c=null;
            int nsent=-1;
            VectorCommand vc=null;       
            //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp); 
            synchronized(IArrived){
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp); 
                //arrived.sem_wait(co);
                while(arrived==0){
                    try { 
                        //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
                        //neosim.kernel.DeadlockTracker.LockDeadlockTracker(co,ecp2); 
                        IArrived.wait();
                        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);
                    }
                    catch(Exception x){
                        x.printStackTrace();
                    }
                }
                arrived--; 
                //neosim.kernel.Deadlock_Message ecp3=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp3); 
                synchronized(co){
                    synchronized(delayed_co){ 
                        //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp3);
            
                if (!terminated_flushed) {

                     nw = nwaiting();
		    /* If >1 commands, bundle them together */
                     if (nw == 1) {
			c = (CommandObj)co.remove( 0 );
                        currentBufferSize-=c.getSize();
                     } else if (nw > 0) {
                        nsent=1;
                        if (debug)  System.out.println("Sending vector command size "+nw);
			CommandObj cmd1 = (CommandObj)co.remove( 0 );
                        currentBufferSize-=cmd1.getSize();
                        if(cmd1 instanceof EventToCommand){
                            ((EventToCommand)cmd1).getEvent();
                        }                            
			vc = new VectorCommand(cmd1.getID());
			if (debug) System.out.println(name+"Doing single command (v):" + cmd1.getType());				
			vc.add(cmd1);
                        debugObject=cmd1;

			Iterator it = co.iterator();
                        debugObject=vc;
			while (co.size()>0) {
                            CommandObj cmd2 = (CommandObj)co.remove(0);
                            currentBufferSize-=cmd2.getSize();                                                  
                            if (debug) System.out.println(name+"Doing single command (v):" + cmd2.getType());

                            vc.add(cmd2);
                            nsent ++;
			 }
                        
                         for (int j=1; j<nsent; j++) { 
                            //arrived.sem_wait(vc);
                            while(arrived==0){
                                try { 
                                    //neosim.kernel.Deadlock_Message ecp4=new neosim.kernel.Deadlock_Message();
                                    //neosim.kernel.DeadlockTracker.LockDeadlockTracker(IArrived,ecp4); 
                                    IArrived.wait();
                                    //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp4);
                                }
                                catch(Exception x){
                                    x.printStackTrace();
                                }
                            }
                            arrived--;                              
                         }                            
                    }
                   
                }
            }}}//sync
            try{
                if (!terminated_flushed) {
                    if (nw == 1) {
			if (debug){ 
                               PrintObject(c,"parallel_debug.txt");
                        }
                        debugObject=c;
                        if(debug_verbose){ //&&nmlplay.RunSim.Running
                            nmlplay.NMLPlay_2 player=(nmlplay.NMLPlay_2)(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player); 
                            if(player!=null&&player.isRunning())
                                PrintObject(c,c.getClass().getName()+".txt");//"parallel_debug.txt"
                        }
                        
                        boolean goon=true;
                        int delay=100;
                        while(goon){
                            try{                        
                                dest.command(c);
                                goon=false;
                            }
                            catch(Exception X){
                                System.out.println("Error_3 in CommandQueue.Add "+dest.getClass().getName());
                                //this.PrintObject(vc,"CommandQueueRun_1_Error.txt");
                                try{
                                   // java.io.PrintWriter trcstream = new java.io.PrintWriter(new java.io.FileOutputStream("CommandQueueRun_1_Error.txt"));
                                   // X.printStackTrace(trcstream);
                                   // trcstream.close();
                                //X.printStackTrace();
                                    Thread.sleep(delay);
                                    delay+=200;
                                }catch(Exception x){x.printStackTrace();}                            
                            }                                
                        }        
                                
			if (debug) System.out.println(name+": Done single command");
			sent.sem_signal();
                    }
                    else if(nw>0){
                        boolean goon=true;
                        int delay=100;
                        while(goon){
                            try{                             
                                dest.command(vc);
                                goon=false;
                            }
                            catch(Exception X){
                                System.out.println("Error_4 in CommandQueue.Add "+dest.getClass().getName());
                                //this.PrintObject(vc,"CommandQueueRun_2_Error.txt");
                                try{
                                    //java.io.PrintWriter trcstream = new java.io.PrintWriter(new java.io.FileOutputStream("CommandQueueRun_2_Error.txt"));
                                    //X.printStackTrace(trcstream);
                                    //trcstream.close();
                                //X.printStackTrace();
                                    Thread.sleep(delay);
                                    delay+=200;
                                }catch(Exception x){x.printStackTrace();}                            
                            }
                        }
                                
                        if(debug_verbose){ //&&nmlplay.RunSim.Running
                             nmlplay.NMLPlay_2 player=(nmlplay.NMLPlay_2)(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player); 
                             if(player!=null&&player.isRunning())                                
                                 PrintObject(vc,vc.getClass().getName()+".txt");	//"parallel_debug.txt"		    
                        }                        
			int j;
			// Ensure correct # of arrived/sent semaphores 

			for (j=0; j<nsent; j++) { 
			    sent.sem_signal();
			}
		    } else {
			if (false)  {
			    System.out.println("CommandQueue: None waiting, but got a signal anyway!");
			}
		    }
		    totalSent +=nw;

		    if (terminated) {terminated_flushed = true; }
                }
            } catch (Exception ex) {
                if (destAlive) 
                    System.out.println("CQ Failed to pass on event\n"+ex.getMessage());
                else
                    System.out.println("CommandQueue: Exception happenned when not Alive");
		ex.printStackTrace();
		destAlive = false;
                PrintObject(debugObject,".\\Exception.txt");
            }            
	}	    
	if (debug)	System.out.println("A Command Queue named "+name+" has finished,"+nwaiting()+" still waiting");
	flush(true);
	if (debug)	System.out.println("A Command Queue named "+name+" has flushed");
        synchronized(nbQueues){
            nbQueues[0]--;
            if(nbQueues[0]!=0)
                MaxBufferSize=TotalBufferSize/nbQueues[0];
        }        
    }
    final static short max_depth=8;
    static final public void PrintObject(Object c, String filepath){
        synchronized(System.out){
            if(c==null)
                System.out.println("Bizarre, got null in neosim.kernel.CommandQueue.PrintObject");
            java.io.FileOutputStream appendedFile;
            try{
                if(filepath==null)
                    appendedFile = new java.io.FileOutputStream("c:\\Zip_October_2004\\jars\\parallel_debug.txt", true);
                else
                    appendedFile = new java.io.FileOutputStream(filepath, true);
                SimKernel MachineId=SimSystem.getKernel();
                String str1=c.getClass().getName().toUpperCase();
                int str2=MachineId!=null?MachineId.getID():-1;
                appendedFile.write(new String("******************************\n\n"+str1+"\tMachineID : "+str2+"\n\n******************************\n\n").getBytes());
                Vector v=new Vector();
                PrintObject(c,0,appendedFile,v);
                appendedFile.write(new String("\n\n").getBytes());                      
                appendedFile.close();        
            }catch(Exception exc){
                exc.printStackTrace();
            }               
        }
    }
    
    static final private void PrintObject(Object o, int tab,java.io.FileOutputStream appendedFile,Vector vect){
        Class c = o.getClass();
        Object fob;
        String muche=new String();                
        for(int j=0;j<tab+1;j++)
             muche+="\t";       
        //implementations of Vectors and Hashtable won't go through !!!
        if(c==java.util.Vector.class){
            int k=((java.util.Vector)o).size();
            for(int l=0;l<k;l++){
                fob=((java.util.Vector)o).elementAt(l);
                if(fob!=null){
                    try{
                      appendedFile.write(new String(muche+"elet "+l+ "\t: "+" "+fob.toString() +"("+fob.getClass().getName()+")\n").getBytes());
                   
                    }catch(Exception exc){
                    }   
 
                      if((tab<max_depth)&&!(fob instanceof Byte)&&!(fob instanceof String)&&!(fob instanceof Integer)&&!(fob instanceof Short)&&!(fob instanceof Double)&&!(fob instanceof Float)&&!(fob instanceof Boolean)&&!(fob instanceof Long)){//required in case it goes to infinite...
                            if(vect.contains(fob)){
                                try{
                                    appendedFile.write(new String(muche+" (described above)").getBytes());
                                }catch(Exception exc){ }
                            }
                            else{
                                vect.add(fob); 
                                PrintObject(fob,tab+1,appendedFile,vect);                    
                            }
                      }
                }
                else{
                    try{
                        appendedFile.write(new String(muche+"elet "+l+ "\t: null(null)\n").getBytes());
                    }catch(Exception exc){
                    
                    }                              
                }
            }    
            return;
        }
        else if(o instanceof List){
            int k=((List)o).size();
            for(int l=0;l<k;l++){
                fob=((List)o).get(l);
                if(fob!=null){
                    try{
                      appendedFile.write(new String(muche+"elet "+l+ "\t: "+" "+fob.toString() +"("+fob.getClass().getName()+")\n").getBytes());
                   
                    }catch(Exception exc){
                    }   
 
                      if((tab<max_depth)&&!(fob instanceof Byte)&&!(fob instanceof String)&&!(fob instanceof Integer)&&!(fob instanceof Short)&&!(fob instanceof Double)&&!(fob instanceof Float)&&!(fob instanceof Boolean)&&!(fob instanceof Long)){//required in case it goes to infinite...
                            if(vect.contains(fob)){
                                try{
                                    appendedFile.write(new String(muche+" (described above)").getBytes());
                                }catch(Exception exc){ }
                            }
                            else{
                                vect.add(fob); 
                                PrintObject(fob,tab+1,appendedFile,vect);                    
                            }
                      }
                }
                else{
                    try{
                        appendedFile.write(new String(muche+"elet "+l+ "\t: null(null)\n").getBytes());
                    }catch(Exception exc){
                    
                    }                              
                }
            }    
            return;
        }        
        else if(c==java.util.Hashtable.class){
            Enumeration enum_keys=((java.util.Hashtable)o).keys();
            Enumeration enum_elements=((java.util.Hashtable)o).elements();
            Object key,element;
            while(enum_keys.hasMoreElements()){
                key=enum_keys.nextElement();
                element=enum_elements.nextElement();
                try{
                    appendedFile.write(new String(muche+"KEY\t: "+" "+key.toString() +"("+key.getClass().getName()+")\n").getBytes());
                    
                }catch(Exception exc){
                    
                }                    
                if((tab<max_depth)&&!(key instanceof Byte)&&!(key instanceof String)&&!(key instanceof Integer)&&!(key instanceof Short)&&!(key instanceof Double)&&!(key instanceof Float)&&!(key instanceof Boolean)&&!(key instanceof Long))//required in case it goes to infinite...
                {
                    if(vect.contains(key)){
                        try{
                          appendedFile.write(new String(muche+" (described above)").getBytes());
                        }catch(Exception exc){ }
                      }
                      else{
                         vect.add(key); 
                         PrintObject(key,tab+1,appendedFile,vect);                    
                      }
                }
                if(element!=null){                    
                    try{
                        appendedFile.write(new String(muche+"ELT\t: "+" "+element.toString() +"("+element.getClass().getName()+")\n").getBytes());                    
                    }catch(Exception exc){
                    
                    }                        
                    if((tab<max_depth)&&!(element instanceof Byte)&&!(element instanceof String)&&!(element instanceof Integer)&&!(element instanceof Short)&&!(element instanceof Double)&&!(element instanceof Float)&&!(element instanceof Boolean)&&!(element instanceof Long))//required in case it goes to infinite...
                            if(vect.contains(element)){
                                try{
                                    appendedFile.write(new String(muche+" (described above)").getBytes());
                                }catch(Exception exc){ }
                            }
                            else{
                                vect.add(element); 
                                PrintObject(element,tab+1,appendedFile,vect);                    
                            }                        
                }
                else{
                    try{
                        appendedFile.write(new String(muche+"ELT\t: null(null)\n").getBytes());
                    }catch(Exception exc){
                    
                    }                        
                    if((tab<max_depth)&&!(element instanceof Byte)&&!(element instanceof String)&&!(element instanceof Integer)&&!(element instanceof Short)&&!(element instanceof Double)&&!(element instanceof Float)&&!(element instanceof Boolean)&&!(element instanceof Long))//required in case it goes to infinite...
                            if(vect.contains(element)){
                                try{
                                    appendedFile.write(new String(muche+" (described above)").getBytes());
                                }catch(Exception exc){ }
                            }
                            else{
                                vect.add(element); 
                                PrintObject(element,tab+1,appendedFile,vect);                    
                            }                     
                }
                try{
                    appendedFile.write(new String(muche+"---------------------------------------\n").getBytes());
                }catch(Exception exc){
                    
                }
                //System.out.println(muche+"---------------------------------------");//separation/
            }
            return;
        }
        else {
            int length=-1;//check if it is an array
            try{
                length=java.lang.reflect.Array.getLength(o);
            }
            catch(Exception e){
            }
            if(length>=0){
                for(int l=0;l<length;l++){
                    fob=java.lang.reflect.Array.get(o, l);
                    if(fob!=null){
                        try{
                            appendedFile.write(new String(muche+"elet "+l+ "\t: "+" "+fob.toString() +"("+fob.getClass().getName()+")\n").getBytes());
                        }catch(Exception exc){
                        }                            
                        if((tab<max_depth)&&!(fob instanceof Byte)&&!(fob instanceof String)&&!(fob instanceof Integer)&&!(fob instanceof Short)&&!(fob instanceof Double)&&!(fob instanceof Float)&&!(fob instanceof Boolean)&&!(fob instanceof Long))//required in case it goes to infinite...
                            if(vect.contains(fob)){
                                try{
                                    appendedFile.write(new String(muche+" (described above)").getBytes());
                                }catch(Exception exc){ }
                            }
                            else{
                                vect.add(fob); 
                                PrintObject(fob,tab+1,appendedFile,vect);                    
                            } 
                    }
                    else{
                        try{
                            appendedFile.write(new String(muche+"elet "+l+ "\t: null(null)\n").getBytes());
                        }catch(Exception exc){
                        }                              
                    }
                }    
                return;                
            }
        }
        
        while(c!=null&&c!=Object.class){        
            Field[] flds=c.getDeclaredFields();
          
            for (int i=0; i<flds.length; i++) {
                try {
                    flds[i].setAccessible(true);
                    fob = flds[i].get(o);
                    if(fob!=null){
                        int mod  = flds[i].getModifiers();
                        if (Modifier.isStatic(mod)) {
                        }
                        else if (Modifier.isTransient(mod)) {
                        }
                        else {
                            String str=flds[i].getType().getName();
                            int k=str.lastIndexOf('.');
                            if(k>0)
                                str=str.substring(k+1);
                            try{
                                appendedFile.write(new String(muche+str+" "+ flds[i].getName()+ "\t: "+" "+fob.toString() +"("+fob.getClass().getName()+")\n").getBytes());
                            }catch(Exception exc){
                    
                            }        
                            if((tab<max_depth)&&!(fob instanceof Byte)&&!(fob instanceof String)&&!(fob instanceof Integer)&&!(fob instanceof Short)&&!(fob instanceof Double)&&!(fob instanceof Float)&&!(fob instanceof Boolean)&&!(fob instanceof Long))//required in case it goes to infinite...
                                if(vect.contains(fob)){
                                    try{
                                        appendedFile.write(new String(muche+" (described above)").getBytes());
                                    }catch(Exception exc){ }
                                }
                                else{
                                    vect.add(fob); 
                                    PrintObject(fob,tab+1,appendedFile,vect);                    
                                } 
                        }
                    }
                    else{
                            String str=flds[i].getType().getName();
                            int k=str.lastIndexOf('.');
                            if(k>0)
                                str=str.substring(k+1);
                            try{
                                appendedFile.write(new String(muche+flds[i].getName()+" null\n").getBytes());
                            }catch(Exception exc){
                    
                            }        
                    }
                    
                }
                catch(Exception e){
                    System.out.println("Exception in PrintObject");
                    e.printStackTrace();
                }
            }
            c=c.getSuperclass();
        }
    }
    public Commandable getDest(){
            return this.dest;
    }    
};

/*
  if(fob instanceof java.util.Vector){
                                    k=((java.util.Vector)fob).size();
                                    Object obj;
                                    for(int l=0;l<k;l++){
                                        obj=((java.util.Vector)fob).elementAt(l);
                                        if(obj!=null){
                                            System.out.println(muche+"    elt "+l+ "\t: "+" "+obj.toString() +"("+obj.getClass().getName()+")");
                                            PrintObject(obj,tab+2);
                                        }
                                        else{
                                            System.out.println(muche+"    elt "+l+ "\t: null(null)");
                                        }
                                    }   
                                }
                                else if(fob instanceof java.util.Hashtable){
                                    Enumeration enum_key=((java.util.Hashtable)fob).keys();   
                                    Enumeration enum_elements=((java.util.Hashtable)fob).elements();
                                    for()
                                }
                                else
 */