package neosim.kernel;
import neosim.iface.*;
import neosim.implementation.*;


import java.util.Vector;

/** A ConnectEvent contains a doOnSysEntity() method which is executed
 * by the default event handler when an event arrives.
 */
public class ConnectEvent extends SysEntEvent {
    Projection p;
 //   neosim.kernel.DeadlockTracker tracker=new neosim.kernel.DeadlockTracker();
    static boolean debug = false;

    public ConnectEvent(Time t, int srce, Projection p, Callback c ) { //EntityID srce
	super( t, srce, PortID.SYSTEM_PORT,c );
	this.p = p;
    }
    public void doOnSysEntity( SystemEntity se )
    {
        this.doOnSysEntity(se,true);
    }
    /** Construct local portion of projection */
    public final void doOnSysEntity( SystemEntity se, boolean BuildConnection ) {
	BasicKernel bk    = (BasicKernel) se.getSimKernel();
        int rank        = bk.getRank();
        int groupsize   = bk.getGroupsize();

	if (debug) { System.out.println("["+rank+"] of "+groupsize+"Connect event, making projection - general?" + 
					p.isGeneralProjection()); }
	if (!p.isGeneralProjection()) {
	    Vector conns = p.getConnections();
	    
	    /** The connections vector should have been filtered out
	     * to just include local entities.
	     */
            //neosim.kernel.Deadlock_Message ecp_=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(se,ecp_);                       
	    for (int i=0; i<conns.size(); i++) {
		ConnectionSpec con = (ConnectionSpec)conns.elementAt(i);
		
		int srceid  = con.getSrcEntityID();//EntityID srceid
		int desteid = con.getDstEntityID();//EntityID desteid
		int srcproc  = bk.getEntityProcessor(srceid);
		int destproc = bk.getEntityProcessor(desteid);
		if (debug) { System.out.println("Trying to connect from "+
						srceid+" to "+desteid+" con "+con); }

		if (srcproc == rank) { 
		    EntityImpl srce = bk.getLocalEntity( srceid );
		    if (debug) { System.out.println("Got source entity "+srce); }
                    //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                    //neosim.kernel.DeadlockTracker.LockDeadlockTracker(con,ecp);                       
		    srce.addOutConnection( con.getSrcPortID(), con.getDelay(),  desteid, con.getDstPortID());
                    //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
		    con.setSrcParams(srce, con.getSrcPortID());
		}

		if (destproc == rank) {
		    EntityImpl deste = bk.getLocalEntity( desteid );
		    if (debug) { System.out.println("Got dest entity "+deste); }
		    ConnectionImpl ci = new ConnectionImpl( srceid, con.getSrcPortID() ,deste, con.getDstPortID() , con.getDelay() );
                    //neosim.kernel.Deadlock_Message ecp=new neosim.kernel.Deadlock_Message();
                    //neosim.kernel.DeadlockTracker.LockDeadlockTracker(ci,ecp);                     
		    int cid=bk.addInConnection( srcproc, ci,BuildConnection ); //Maybe something to do here with the boolean : shoulde be fase in case of disconnect event
                    //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp);
		    if(BuildConnection)
                        con.setDstParams(deste,cid);//new ConnectionID(ci.getConnectionID()
		}
	    }

	    if (debug) { System.out.println("Connect event, sending callback"); }

  
	    sendCallback( se, new Integer(42) );
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp_);            
	} else {

	    // This is a GeneralProjection with src/dst methods
            //neosim.kernel.Deadlock_Message ecp0=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp0);    
                
	    GeneralProjection gp = (GeneralProjection) p;
	    String srcPopName = gp.getSrcPopName();
	    String destPopName = gp.getDestPopName();
	    PopulationTree root = bk.getPopulationTree( getTime() );

	    PopulationImpl sp = (PopulationImpl)root.getChild(srcPopName);
	    PopulationImpl dp = (PopulationImpl)root.getChild(destPopName);

	    if (debug) { System.out.println(rank+" General Projection from "+srcPopName+" to "+destPopName); }

	    SourceMethod sm = gp.getSrcMethod();
	    DestMethod dm = gp.getDestMethod();
	    
	    Vector srcs = sp.getLocalMembers();
            int size=0;
            if(srcs!=null)
                size=srcs.size();
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp0);   
  
	    for (int i=0; i<size; i++) {
                //neosim.kernel.Deadlock_Message ecp1=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(sm,ecp1);                 
		Entity srce = (Entity)srcs.elementAt(i);                                  
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp1);               
                //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
                //neosim.kernel.DeadlockTracker.LockDeadlockTracker(sm,ecp2);                                 
		sm.sendRequests( srce, dp, dm,  BuildConnection );
                //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);               
	    }
 
            //neosim.kernel.Deadlock_Message ecp2=new neosim.kernel.Deadlock_Message();
            //neosim.kernel.DeadlockTracker.LockDeadlockTracker(this,ecp2);               
	    if (debug) { System.out.println(rank+" General Projection from "+srcPopName+" to "+destPopName+" done"); }	    
	    // Only send callback when all requests have come in...
	    // Waiting for srcs.size() done msgs to come in....
  
	    sendCallback( se, new Integer(42) );
            //neosim.kernel.DeadlockTracker.UnLockDeadlockTracker(ecp2);  
            
           
	}
    }
    
    public ClassID registerClass(NeosimClasses nc) { 
	cid = nc.registerClass("ConnectEvent", "SysEntEvent", "neosim.kernel"); return cid; 
    }
    
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {
        super.readExternal(objectInput);
       // System.out.println("ConnectEvent: readObject  ");
        p=(Projection)objectInput.readObject();
        // System.out.println("ConnectEvent: readObject  2");
    }
    
    public ConnectEvent(){
        super();
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        super.writeExternal(objectOutput);
        objectOutput.writeObject(p);
    }  
    
    public int getSize(){
        return 100;
    }    
}
