package neosim.kernel;
import neosim.iface.*;


/** 
 * Connection query event is used to query an entity
 * about its connections.
 */
public class ConnectionQueryEvent extends EntityEvent {
    ConnectionQuery cq;
    Callback cb;
    int oute;// EntityID oute;
    short outp;//PortID outp

    Connection qc;

    /** The originating processor
     */
    protected int proc; // originating proc
    int qid;


    /**
     * @param t the event time
     * @param cbe the callback entity
     * @param cbport the callback port
     * @param cq the query to run on each connection
     * @param oute the output entity
     * @param outp the output port
     */
    public ConnectionQueryEvent(Time t, int cbe, short cbport, //Time t, EntityID cbe, PortID cbport,//PortID cbport
				ConnectionQuery cq, int oute, short outp, Callback cb ) { //EntityID oute, PortID outp,//PortID outp
	super( t, cbe, cbport );
	this.oute = oute;
	this.outp = outp;
	this.cq = cq; 
	this.cb = cb;
	qc = null;
    }

    public void setQid(int qid) { this.qid = qid; }
    public int getQid() { return qid; } 
    public void setProc(int proc) { this.proc = proc; }
    public int getProc() { return proc; } 
    public Callback getCallback() { return cb; }

    public int getOutputEntity() { return oute; }//EntityID return
    public short getOutputPort() { return outp; }//PortID

    /** The Query connection is the connection which is queried: 
     * the event actually arrives on a special connection from the System entity 
     */
    public void setQueryConnection( Connection qc ) { this.qc = qc; }

    public void doOnEntity( Entity e ) {
	//	System.out.println("Doing connection query event");

	Object ans = cq.query(e, qc);

	//	System.out.println("Done connection query");
	sendCallback( ans );
	//	System.out.println("Done send response");
    }


    /** Send a callback to originator, containing the answer, 
     * Note this is ripe for an optimisation based on 
     * running connection query on all local entities.
     * @param ans the answer to return
     */
    public void sendCallback( Object ans ) {
	SimKernel sk = SimSystem.getKernel();

//JAG DANGEROUS OPTIMIZATION         
        //sk.sendCommand( proc, new QueryResponse( proc, qid, ans ) );
        QueryResponse qr ;
        if(ans==null){
            qr = new QueryResponseNull(proc, qid, ans);
        }
        else{
            Class cls=ans.getClass();
            if(cls==neosim.iface.Time.class){
                qr = new QueryResponseTime(proc, qid, ans);
                ((QueryResponseTime)qr).index=4;
            }
            else if(cls==Integer.class)
                qr = new QueryResponseInteger(proc, qid, ans);
            else
                qr = new QueryResponse(proc, qid, ans);
        }
        sk.sendCommand( proc, qr );
        
	
        
    }


    public Object clone() { 
	ConnectionQueryEvent cqe = new ConnectionQueryEvent( getTime(), getSrcEntityID(), getSrcEntityPortID(), 
							     cq, oute, outp, cb ); 
	cqe.setProc(proc);
	cqe.setQid(qid);
	return cqe;
    }
    
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {
        super.readExternal(objectInput);
        oute=objectInput.readInt();
        outp=objectInput.readShort();
        proc=objectInput.readInt();
        qid=objectInput.readInt();
        cb=(Callback)objectInput.readObject();
        cq=(ConnectionQuery)objectInput.readObject();
        qc=(Connection)objectInput.readObject();
    }
    
    public ConnectionQueryEvent(){
        super();
        oute=-1;//new EntityID(-1);
        outp=-1;//new PortID(
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        super.writeExternal(objectOutput);
        objectOutput.writeInt(oute);
        objectOutput.writeShort(outp);
        objectOutput.writeInt(proc);
        objectOutput.writeInt(qid);
        objectOutput.writeObject(cb);        
        objectOutput.writeObject(cq); 
        objectOutput.writeObject(qc); 
    }
    
    public int getSize(){
        return 200;//no idea but looks big with 3 objects
    }    
}
