package neosim.kernel;

import neosim.iface.*;
import neosim.implementation.*;


/** A ConstructEvent contains a doOnSysEntity() method which is executed
 * by the default event handler when an event arrives.
 * It is the superclass for kernel operations to make/delete connections
 */
public class ConstructEvent extends SysEntEvent {
    PopulationBuilder pb;
    static private boolean debug = false;

    public ConstructEvent(Time t, int srce, PopulationBuilder pb, Callback c ) { //EntityID srce
	super( t, srce, PortID.SYSTEM_PORT, c );
	this.pb = pb;
    }
    
 //   public String toString(){
 //       System.out.println(super.toString());
 //       System.out.println(pb.toString());
 //       return new String();
 //   }

    /** Construct local portion of population */
    public final void doOnSysEntity( SystemEntity se ) {
        if(pb.getEntityClass().IsInParent()){
//JAG Build some stuff on the constructing proc     
            
            int totalcells  = pb.getNumEnts();
            BasicKernel sk  = (BasicKernel)se.getSimKernel();
            int rank        = sk.getRank();
            int groupsize   = sk.getGroupsize();
            int first = sk.allocateIDs(totalcells);//EntityID first
            PopulationTree pt = sk.getPopulationTree(getTime());
            pt.addNode( pb ); 

            
            for (int j = 0; j < totalcells ; j++) {
               int newe = first + j ;//EntityID newe= new EntityID( first.id + j );
               sk.addRemoteEntity( newe,  this.proc );	//p   
               pb.addMember( newe );
            }
            System.out.println("Proc "+rank+" Building proc "+proc);
            if(proc==rank){
                for (int i=0; i<totalcells; i++) {  
                     EntityClass ec = pb.getEntityClass();
                    Entity e = ec.makeEntity(pb,i);
                    EntityInit ei = pb.getEntityInit();
                    if (ei!=null) { ei.init(e, pb, i); }
                    ((PopulationBuilderImpl)pb).addLocalMember( e );
                }
            }
            sendCallback( se, new Integer(42) );
        }
        else{
            int totalcells  = pb.getNumEnts();

            BasicKernel sk  = (BasicKernel)se.getSimKernel();
            int rank        = sk.getRank();
            int groupsize   = sk.getGroupsize();


            int first = sk.allocateIDs(totalcells);//EntityID first
            int last = first+totalcells-1;
            if (debug)     System.out.println("["+rank+"] Creating entities from "+first+" to "+last);

            PopulationTree pt = sk.getPopulationTree(getTime());
            pt.addNode( pb ); 

            if (debug)     System.out.println("["+rank+"] Calculating local slice length "+
					  totalcells+" "+rank+" "+groupsize);

        // Do split to work out how many cells are local (assuming divisible)
            int localncells = getSliceLength(totalcells,rank,groupsize);
            int numents     = totalcells;

        // Basic starti calculation
            int starti      = getStartOfSlice(numents, rank, groupsize);
 
	/** Build the map of all entity handles->processors 
	 * NB this should be done before making the entities, as the
	 * entities use the population to set their handles.
	 */
            int j=0;
            for (int p=0; p<groupsize; p++) {
                for (int sli = 0; sli < getSliceLength( totalcells,p,groupsize); sli++, j++) {
                    int newe = first + j ;//EntityID newe = new EntityID( first.id + j );
                    sk.addRemoteEntity( newe,  p );	//p   
                    pb.addMember( newe );
                }
            }

	/** Create local entities */
            if (debug)     
                System.out.println("["+rank+"] Creating local map : "+localncells+" starting at "+starti+" out of "+totalcells);
            for (int i=0; i<localncells; i++) {  
                int globi = i + starti;
                if (debug) {	    System.out.println("["+rank+"] About to make entity "+i); }

                EntityClass ec = pb.getEntityClass();

	    //	    if (debug) {	    System.out.println("["+rank+"] Got class for entity "+i); }

                Entity e = ec.makeEntity(pb,globi);

                if (debug) {	    System.out.println("["+rank+"] About to perform entity init "); }

                EntityInit ei = pb.getEntityInit();
                if (ei!=null) { ei.init(e, pb, globi); }

	    // Maintain vector of local members in the population
                ((PopulationBuilderImpl)pb).addLocalMember( e );
	    // Add to kernel...
            }

            if (debug) {	System.out.println("ConstructEvent Sending Callback"); }
            sendCallback( se, new Integer(42) );
	//	System.out.println("["+rank+"] done on sys entity");
            if (debug)	System.out.println("ConstructEvent Done Its Stuff");
        }
    }

    /** Allocation of entities to processors
     * Version which allocates each processor (length DIV nslices),
     * and the 1st (length MOD slices) slices get an extra one each
     */
    //equal dist between proc including rendering starion
    public final int _getSliceLength(int length, int slice, int nslices) {
        int ldiv = length / nslices;
        int lmod = length - (ldiv*nslices);
        int ret = ldiv;
        if (slice < lmod) ret = ret + 1;                             
        return ret;
    }

    /** Get global start index of slice 
     */
    public final int _getStartOfSlice( int length, int slice, int nslices) {
        int ldiv = length / nslices;
        int lmod = length - (ldiv*nslices);
        /* Extra is smaller of slice, lmod */
        int extra = slice;
        if (lmod < slice) extra = lmod;
        return (slice*ldiv) + extra;
    }
    
    public final int getSliceLength(int length, int slice, int nslices) {
        if(nslices!=1){
            if(nslices==slice+1){
                return 0;
            }
            else{
                int ldiv = length / (nslices-1);
                int lmod = length - (ldiv*(nslices-1));
                int ret = ldiv;
                if (slice < lmod) ret = ret + 1;                             
                return ret;                
            }
        }
        int ldiv = length / nslices;
        int lmod = length - (ldiv*nslices);
        int ret = ldiv;
        if (slice < lmod) ret = ret + 1;                             
        return ret;
    }

    /** Get global start index of slice 
     */
    public final int getStartOfSlice( int length, int slice, int nslices) {
        if(nslices!=1){
            if(nslices==slice+1){
                return length;
            }        
            int ldiv = length / (nslices-1);
            int lmod = length - (ldiv*(nslices-1));
            /* Extra is smaller of slice, lmod */
            int extra = slice;
            if (lmod < slice) extra = lmod;
            return (slice*ldiv) + extra;
        }
        int ldiv = length / nslices;
        int lmod = length - (ldiv*nslices);
        /* Extra is smaller of slice, lmod */
        int extra = slice;
        if (lmod < slice) extra = lmod;
        return (slice*ldiv) + extra;        
    }    


    public final ClassID registerClass(NeosimClasses nc) { 
	cid = nc.registerClass("ConstructEvent", "SysEntEvent", "neosim.kernel"); return cid; 
    }
    
     public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {
        super.readExternal(objectInput);
       // System.out.println("ConstructEvent: readObject  ");
        pb=(PopulationBuilder)objectInput.readObject();
       // System.out.println("ConstructEvent: readObject  2");
    }
    
    public ConstructEvent(){
        super();
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        super.writeExternal(objectOutput);
        objectOutput.writeObject(pb);
    }    
    public int getSize(){
        return 200;
    }    
}
