package neosim.kernel;

import java.util.Vector;

/** An OutPortTable (One for each output port)
 * Contains a vector of destination processors
 */
public class OutPortTable implements java.io.Serializable {
    Vector dests;	/* Vector of OutPort */

    public OutPortTable() {
	dests = new Vector();
    }

    /** Add a new output connection 
     */
    public final void add(int proc) {
	OutPort p = getOutPort(proc);
	if (p==null) {
	    p = new OutPort(proc);
	    dests.addElement(p);
	} else {
	    p.addConnection();
	}
    }

    /** Remove an output connection
     */
    public final void remove(int proc) {
	OutPort p = getOutPort(proc);
	if (p==null) {
	    // No connection to that processor, do nothing
	} else {
	    p.removeConnection();
	    if (p.numConnections() <= 0) {
		delete(p);
	    }
	}
    }
    
    /** Get the outport to processor proc, or null if not present
     */
    public final OutPort getOutPort( int proc ) {
	OutPort ret = null;
	for (int i=0; i<dests.size(); i++) {
	    OutPort p =  (OutPort)dests.elementAt(i);
	    if (p.getProc() == proc) { ret = p; }
	}
	return ret;
    }

    /** No more connections to a processor, delete the entry
     */
    private final void delete(OutPort dp) {
	for (int i=0; i<dests.size(); i++) {
	    OutPort p =  (OutPort)dests.elementAt(i);
	    if (p.getProc() == dp.getProc()) { dests.removeElementAt(i); }
	}	
    }

    
    /** Returns total number of output connections from this port */
    public final int numConnections() {
	int n =0;
	for (int i=0; i<dests.size(); i++) {
	    OutPort p =  (OutPort)dests.elementAt(i);
	    n+=p.numConnections();	    
	}
	return n;
    }

    public final Vector getPorts() {
	return dests;
    }
  
}
