/* SimOutfile.java
 */

package neosim.kernel;


import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * The default trace output class, which implements the Sim_output
 * interface. This class implements output to a file called tracefile
 * in the current directory, and is used by default in full Java applications.
 * Note that it can't be used in applets, because they do not allow access
 * to files.
 *
 * Updated for JDK1.1 to use PrintWriter not a PrintStream.
 * @see         SimOutput
 * @version     1.0, 4 September
 * @version     1.1, 24 March 1997
 * @version     1.2  13 November 2001 * Paul Rogister : modify not to crash applets
 * @author      Ross McNab, Fred Howell
 */

public class SimOutfile implements SimOutput {
  private PrintWriter trcstream;
    String fname;
  /** Constructor */
  public SimOutfile(String fname)  {this.fname=fname; }
  /** Attempts to open the file */
  public final void initialise() {
    try {
      trcstream = new PrintWriter(new FileOutputStream(fname));
    } catch(IOException e) {
      System.out.println("SimSystem: Error - could not open trace file");
    } catch(java.security.AccessControlException ae) {
	//System.out.println("SimSystem: Error - could not open trace file");
	trcstream = null;
    }
  }
  /** Prints a trace line to the file
   * @param msg The message to print
   */
  public final void println(String msg) {
    if (trcstream!=null) trcstream.println(msg);
    flush(); /** flush after every line (slow) */
  }

  /** Closes the file */
  public final void close() {
    if (trcstream!=null) trcstream.close();
  }
  /** Flush the output */
  public final void flush() {
    if (trcstream!=null) trcstream.flush();
  }
}
