package neosim.kernel;
import neosim.iface.*;


/** A SysEntEvent contains a doOnSysEntity() method which is executed
 * by the default event handler when an event arrives.
 * It is the superclass for kernel operations to make/delete connections
 */
public class SysEntEvent extends Event  {
    protected int proc; // originating proc
    protected int qid;  // originating query id
    protected Callback c;

    static boolean debug = false;
    
    public SysEntEvent(){
        super();
    }

    public SysEntEvent(Time t, int srce, short srcport, Callback c ) {// EntityID srce, PortID srcport
	super( t, srce, srcport );
	this.c =c;
    }
    public final String toString(){
        System.out.println(super.toString());
        return new String(" proc "+proc+" qid "+qid);
    }
    
    public final void setProc(int proc) { this.proc = proc; }
    public final int getProc() { return proc; } 
    public final void setQid(int qid) { this.qid = qid; }
    public final int getQid() { return qid; } 
    public final Callback getCallback() { return c; }
    public final void setCallback(Callback c) { this.c = c; }

    /** This method is overridden in derived events. 
     */
    public void doOnSysEntity( SystemEntity e ) {
	// A null place holder.
	// QueryResponse qr = new QueryResponse(sp.getID(), qid, ret);
	System.out.println("Error - SysEntEvent called");
	sendCallback( e, new Integer(42) );
    }

    /** Send a callback to originator 
     */
    public final void sendCallback( SystemEntity e, Object ans ) {
	SimKernel sk = e.getSimKernel();
	if (debug) System.out.println("System entity sending a response");
//JAG DANGEROUS OPTIMIZATION         
       // sk.sendCommand( proc, new QueryResponse( sk.getID(), qid, ans ) );
        QueryResponse qr;
        if(ans==null){
            qr = new QueryResponseNull(proc, qid, ans);
        }
        else{
            Class cls=ans.getClass();
            if(cls==neosim.iface.Time.class){
                qr = new QueryResponseTime(sk.getID(), qid, ans);
                ((QueryResponseTime)qr).index=3;
            }
            else if(cls==Integer.class)
                qr = new QueryResponseInteger(sk.getID(), qid, ans);
            else
                qr = new QueryResponse(sk.getID(), qid, ans);
        }
        sk.sendCommand( proc, qr );      
	
	if (debug) System.out.println("System entity sent a response!");
    }

    public ClassID registerClass(NeosimClasses nc) { 
	cid = nc.registerClass("SysEntEvent", "Event", "neosim.kernel"); return cid; 
    }
    public final boolean      isSystemEvent() { return true; }
    
    public void readExternal(java.io.ObjectInput objectInput) throws java.io.IOException, java.lang.ClassNotFoundException {
        super.readExternal(objectInput);
//        System.out.println(this.getClass().getName()+"In SysEntEvent: readObject  ");
        proc=objectInput.readInt();
        qid=objectInput.readInt();
        c=(Callback)objectInput.readObject();
 //       System.out.println(this.getClass().getName()+"In SysEntEvent: readObject  2");
    }
    
    public void writeExternal(java.io.ObjectOutput objectOutput) throws java.io.IOException {
        super.writeExternal(objectOutput);
        objectOutput.writeInt(proc);
        objectOutput.writeInt(qid);
        objectOutput.writeObject(c);        
    }        
    
    public int getSize(){
        return 100;// object can be small but can be really big too
    }     
}
