package neosim.neuroml;

import neosim.iface.*;
import neosim.implementation.*;
import java.util.Hashtable;
import java.util.*;
import neuroml.sim.run.*;
import neuroml.sim.state.*;
import neuroml.model.MeasurementTool.*;
/**
 * Neosim interface to Neuroml entity
 * @author Paul Rogister
 * @date Aug 2001
 */
public final class NeosimEntityImpl extends EntityImpl implements SpikeHandler, ObjectHandler, neuroml.sim.run.SpikeSender, PositionState, neuroml.sim.run.MeasurementHandler {
    neuroml.sim.run.RunnableCell rc;
    neuroml.sim.state.Shape s;
    SpikeIndexTable spTable;//JAN05 NEED to set up    
    

    
    private static boolean debug = false;
       
    public NeosimEntityImpl(EntityClass ec, Population p, int index) {
	super(ec,p,index);	
        spTable=new SpikeIndexTable();        
    }

    public void setRunnableCell(RunnableCell rc) {
	this.rc = rc;
    }

    public RunnableCell getRunnableCell() {
	return rc;
    }

    public void reset() { 
	super.reset(); // *** Required ***
	rc.reset();
    }

    public void advance( Time endt ) {
        //JAN05 CHANGE 
        //rc.advance(endt.s());	    
        spTable.advance(rc, endt.t);
        if(endt.t!=0)
            endt.t=endt.t;
	
    }

    /** Pass events from Neosim to NeuroML versions
     */
    public void handle(Time t, SpikeEvent e, Input i) {
	rc.handleSpike(t.s(),i.getConnection().getConnectionID());	
    }

    /** Implementation of SpikeSender
     */
    public void sendSpike(double t, short outputConnection) {	
        //JAN05
	//sendEvent(outputConnection, new SpikeEvent(new Time(t)) );//new PortID(
        sk.addSpike(getID(),outputConnection,Time.convert(t));
    }

    /** Pass object events from Neosim to NeuroML versions
     */
    public void handle( Time t, ObjectEvent e, Input i) {
	rc.handle(e.getObject(), t.s(),i.getConnection().getConnectionID());	
    }

    /** Send an object on a given output 
     */
    public void send(Object o, double t, short outputConnection){
	sendEvent( outputConnection, new ObjectEvent(new Time(t), o));//new PortID(
    }

    /** Set dest connection params...
     * @deprecated
     */
    public void setDstConnectionParams(Connection c, int destcid, Hashtable params) {//ConnectionID destcid
	/**
	   //  DON'T DO IT THIS WAY !!! 
	NeuromlConnectionState cState = new NeuromlConnectionState(c);
	if (debug) System.out.println("NeosimEntityImpl call setInputConnectionParameter");
	rc.setInputConnectionParameters(destcid.getValue(), null, null);
	**/
    }
    
    public void setPos(double x, double y, double z){
        if(rc instanceof PositionState)
            ((PositionState)rc).setPos(x,y,z);
    }
    public double getXPos(){
       return ((PositionState)rc).getXPos();      
    }
    public double getYPos(){
       return ((PositionState)rc).getYPos();      
    }
    public double getZPos(){
       return ((PositionState)rc).getZPos();      
    }    
    public void setShape(neuroml.sim.state.Shape s )
    {
        this.s=s;
    }
    public neuroml.sim.state.Shape getShape(){
        return s;
    }

    public void draw(){
        s.draw();
    }
    /**The entity sending measurement*/
    public void setMeasured(MeasurementCollection MeasurementList){
        rc.setMeasured(MeasurementList);
    }
    /**The entity receiving measurement*/
    public void setMeasured(int SrcId, MeasurementCollection MeasurementList){
        queryEntity(getLocalTime(), SrcId, new MeasurementQuery(MeasurementList),//new EntityID(SrcId)
					  null );
    }
    
    //pure debug stuff
    //int NbSpike=0;    
    public void addSpike(int index,long time){
         spTable.addSpike(index,time);
      /*   NbSpike++;
         if(this.handle==6){
             if((NbSpike&0x1F)==0)//modulo 32==0
                 System.out.print("handle 6 nbSpikes: "+NbSpike);
         }*/
    }
 /** This method sends an event on output port c.
      *  It will not be sent immediately but only when a CommandQueue is about to be flushed after each simulation time step (time step which allows pausing the simulation NOT the time step corresponding to the minimal delay). 
      *  This method is meant for some measurements, in order to ensure that the amount of communications requested by measurements does not become a limiting factor for. 
     * @param handle The originating entity.
     * @param c The output port to send the event from.
     * @param e The event to send. Its timestamp should
     * be the <em>sending time</em>
     */    
    public void sendDelayed(Object o, double t, short outputConnection){
        sendEvent( outputConnection, new ObjectEvent(new Time(t), o, true));
    }
}
