
package neosim.neuroml;

import neuroml.sim.state.*;
import neuroml.util.module.*;
import neosim.iface.*;

import java.util.Vector;
import java.util.ArrayList;

/**  A population is a collection of one or more entity handles.
 * It can contain extra methods for initialising the elements of
 * the population, or for indexing them rapidly.
 */
public final class NeuromlPopulationState extends BasicPopulationState {
  
    neosim.iface.Population np;

    public NeuromlPopulationState( neosim.iface.Population p ) { 
        if(p==null)
            p=null;
        np = p; 
        setName(p.getName()); 
    }

    /** Returns number of elements */
    public int getNumElements() { return np.getNumEnts();}

    /** Returns unique population id */
    public int getID() { return np.getID(); }

    /** Set the population id    */
    public void setID(int id){  np.setID(id); }

    /** Returns index of a given entity ID
     * @param eid handle of entity
     * @return index of entity in population if present, or -1 otherwise.
     */
    public int getIndex( int eid ) { return np.getIndex( eid); }//new EntityID(eid))

    /** Returns entityID of a given index
     * @param index index in population
     * @return entityID of given index
     */
    public int getElementID(int index) { 
	int res = np.getEntityID(index ); //EntityID res
	return res;//.getValue();
    }
    
    /** Returns whether a given entity is a member
     * @param eid handle of entity
     * @return true if entity is a member of this population
     */
    public boolean isMember(int eid) { return np.isMember( eid); }//new EntityID(eid)

    /** Returns a list of all members 
     * - a list of Integers
     * WARNING this implementation does a copy.
     */
    public java.util.List getElementIDs() { 
	Vector v = np.getMemberList(); 
	java.util.List ret = new ArrayList();
	for (int i=0; i<v.size(); i++) {
	    ret.add((Integer) (v.elementAt(i)) );//((EntityID)v.elementAt(i)).id
	}
	return ret;
    }

    /** Return a list of names
     */
    public java.util.List getElementNames() { 
	Vector v = np.getMemberList(); 
	java.util.List ret = new ArrayList();
	for (int i=0; i<v.size(); i++) {
	    ret.add( getName()+"["+i+"]" );
	}
	return ret;	
    }

    /** Add a member to this population
     * @param eid handle of entity
     */
    public void addElement(int eid, String n) { np.addMember(eid); }//new EntityID(eid))
}
