//package neosim.neuroml.nmllauncher;
package neosim.util.loaders.nmllauncher;

/**
 * Launches a Neosim simulation from a XML description file of that simulation
 *
 * Parser -> Sax -> SaxHandler -> SimDesc -> Simlauncher
 * 
 * Paul Rogister
 * 28/09/2000
 * 
 * Modified by fwh oct 24 00 to not call initSim() & terminate() from 
 * bootstrap, for parallel version.
 */

import neuroml.util.*;
import neuroml.sim.run.*;

import neuroml.sim.state.*;

import neosim.kernel.*;
import neosim.iface.*;
import neosim.implementation.*; 
import neosim.util.BasicApp; 
import neosim.neuroml.*;
import java.beans.*;


import java.util.Random;
import java.util.Vector;
import java.lang.Math;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.File;
import java.net.*;
import java.io.*;



/**
 * Call the parser then the launcher of Neosim simulation.
 *
 * There are two modes of operation:
 *   1) just give a complete xml file and run it
 *        this can do complex things using the controlled schedule modules
 *   2) interactive access, where we want to
 *        give model as object
 *        build model  
 *        run for time
 *        reset
 *        query
 *
 * @author Paul Rogister and Fred Howell
 *
 * @version 1.0, 18 October 2000     
 *                 
 */
public final class XML_Reader implements Simulator {
    
    /** The real simulator... 
     * - a view of the ScriptReader
     */
    neuroml.sim.run.Simulator sim;
    protected BasicApp ba;

    protected static Kernel sk;

    // Simulation model
    private Object model; 
    String args[];
    // to launch the simulation from the description
    protected NeosimNeuroMLLauncher simlauncher;
  
    static boolean debug = false;
    boolean modelIsBuilt = false;

    // public field as parameter of the simulator
    public SeqKernelParams kernelParams = new SeqKernelParams();


    /** Construct XML Reader 
     */
    public XML_Reader() { 
	ba = new BasicApp();
	sim = new NeuromlSimulator( ba ); // View of Neosim script reader
    }   

    /** gets the model.
     */
    public final Object getModel(){
	return model;
    }

    /** Initialise the simulation kernel 
     */
    public final void initKernel( String args[] ) {
	sk = ba.initKernel(args);
	ba.bootstrap(args);
    }

    /** Parameter the simulation 
     */
    public final void setParameters(String args[]){
	this.args = args;

    }
    /**
     * Launch a simulation from a Simulation description fullfilled by the parser
     *
     * @param simdesc Simulation description
     *                 
     */
    public void launchSim( Object simdesc  ) {
	launchSim(  simdesc, null );
    }

    public void launchSim( Object model, Namespace namespace ) {
	simlauncher = new NeosimNeuroMLLauncher(this,model,namespace);
	// check the Neosim version
	//simlauncher.checkVersion();
	// create and initialise all the prototype entities
	if (debug) { System.out.println("XML_Reader : Load prototypes"); }
	simlauncher.loadPrototypes(); // call jarload
	// build the model, populations and projections
	if (debug) { System.out.println("XML_Reader : Build model"); }
	simlauncher.buildModel(); // zero in order to build the main network
       
	// run the sim
	if (debug) { System.out.println("XML_Reader : Run simulation"); }
	simlauncher.runSimulation();
	// simlauncher.runqueries(); //deal with the queries later
    }
    
    /** Build and run simulation 
     * Used to initialise kernel, run a model from an XML file.
     */
    public void bootstrap(String args[]) {
	ba.bootstrap(args);
	System.out.println("======= Neosim XML NeuroML Reader 1.2 July 2001 ======");
	if (debug) {  System.out.println("NeuroML Reader starting..."); }
	// last arg is used to specify the XML file.
	// previous args are used to initialise the kernel (e.g. thr 2, seq)
        if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Load XML file "+args[args.length-1]);         
	model = XMLIn.loadXML( args[args.length-1] );
	if (model==null) {
	    System.out.println(" NEOSIM NeuroML Reader : Error NeuroML file "+args[args.length-1]+
			       " not readable");
	    return;
	}
	Namespace namespace = NeuromlSys.getNamespace(args[args.length-1]);
        if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("");
	launchSim(model,namespace);	
	System.out.println("=== NEOSIM NeuroML Reader : done. Bye! ===");
    }  

    /** Launch routine which allows the XML_Reader to run as
     * a module.  Loads an XML file.
     */
    public void launch(String args[]) {
	if (debug) { System.out.println("=== NEOSIM -XML Reader as NeuroML Simulator Module === "); }

	//initKernel(args);
	//	if (ba.needToConstruct()) {


	if (modelIsBuilt && (simlauncher!=null)) {
	    simlauncher.runSimulation();

	} else {
	    initKernel(args);
	    bootstrap(args);
	}
	ba.destroyKernel();
	    //	}
    }  
    
    /** setParameters
     * set the args parameter according to the simulator public parameters
     * here, the parameters are either :
     * seq <filename>
     * thr <nthread> <filename>
     * rmi [-c] <server> <user> <nprocs> <nthread>
     */
    String[] getParameters(){
	
	if (args == null){

	    Integer itemp;
	    if ( kernelParams instanceof ThrKernelParams){
		ThrKernelParams thrkernelParams = (ThrKernelParams)kernelParams;
		args = new String[3];
		args[0] = "thr";
		itemp = new Integer(thrkernelParams.nThread);
		args[1] = itemp.toString();
		args[2] = thrkernelParams.neuromlFile.toString();
	    } else 	if ( kernelParams instanceof RmiKernelParams){
		RmiKernelParams rmikernelParams = (RmiKernelParams)kernelParams;
		
		int inc = 0;
		if (rmikernelParams.buildMode.equals("-c")){
		    args = new String[7];
		    args[1] = rmikernelParams.buildMode;
		    inc = 1;		
		} else {
		    args = new String[6];
		}
		args[0] = "rmi";
		
		args[1+inc] = rmikernelParams.server;
		args[2+inc] = rmikernelParams.user;
		itemp = new Integer(rmikernelParams.nProcs);
		args[3+inc] = itemp.toString();
		itemp = new Integer(rmikernelParams.nThread);
		
		args[4+inc] = itemp.toString();
		args[5+inc] = rmikernelParams.neuromlFile.toString();
		
	    } else 	if ( kernelParams instanceof SeqKernelParams){
		args = new String[2];
		args[0] = "seq";
		args[1] = kernelParams.neuromlFile.toString();		
	    }	    
	}
	return args;
    }


    /** Launch routine which allows the XML_Reader to run as
     * a module
     */
    public void launch() {
	if (debug) { System.out.println("=== NEOSIM -XML Reader as NeuroML Simulator Module === "); }

	// set the args parameter according to the simulator public parameters
	// here, the parameters are either :
	// seq <filename>
	// thr <nthread> <filename>
	// rmi [-c] <server> <user> <nprocs> <nthread>
	String args[] = getParameters();

	SimKernel sk = ba.initKernel(args);
	//	if (ba.needToConstruct()) {
	bootstrap(args);
	ba.destroyKernel();
	//	}
    }  

    /** Main routine which allows the XML_Reader to run as
     * an application
     * (an alternative is to use the Jarload to load it)
     */
    public static void main(String args[]) {
	if (debug) { System.out.println("=== NEOSIM - XML Reader as Application === "); }
	XML_Reader ap = new XML_Reader();
	SimKernel sk = ap.ba.initKernel(args);
	ap.bootstrap(args);

    }  


    /**** The following Simulator methods are redirected to the ScriptResder via
     * a NeuromlSimulator converter
     *  ---- Should this module implement a different interface from Simulator???
     */


    /** Initialise the simulation - called automatically
     */
    public void initSim(){
	sim.initSim();
    }

    /** Build a model from a NeuroML object description
     */
    public void buildModel( Object model, Namespace ns ) {
	if (model==null){
	    String args[] = getParameters();
            if(neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player!=null)
                neosim.util.loaders.nmllauncher.NeosimNeuroMLLauncher.Player.println("Initialize the kernel");
	    ba.initKernel(args);         
	    ba.bootstrap(args);
	    
	    // last arg is used to specify the XML file.
	    // previous args are used to initialise the kernel (e.g. thr 2, seq)
	    
	    model = XMLIn.loadXML( args[args.length-1] );
	    if (debug) System.out.println("buildModel model "+model+" for param "+args[args.length-1]);

	    if (model==null) {
		System.out.println(" NEOSIM NeuroML Reader : Error NeuroML file "+args[args.length-1]+
				   " not readable");
		return;
	    }
	    ns = NeuromlSys.getNamespace(args[args.length-1]);

	}
	this.model = model;
	// initKernel(args);
        //simlauncher = new NeosimNeuroMLLauncher(null,null,null);
	simlauncher = new NeosimNeuroMLLauncher(this,model,ns);
        simlauncher.loadPrototypes();
	simlauncher.buildModel();
	modelIsBuilt = true;
        sk.runSim(new Time(1e-6));//18/04/05
       /* System.out.println("Doing barrier");
        ((SimKernel)sk).barrier();*/
	//	simlauncher.runsimulation();		
    }

    /** Run the simulation until the given time
     * @param t the time to run for.
     */
    public void runSim(double t) {
	run(t);
    }

    public void run(double t) {
	if (debug) System.out.println("XML_Reader told to advance for "+t+" sim is "+sim);
	sim.runSim(t);
    }
    
    public void reset(){
	sim.reset();
    }

    public void terminate(){
	sim.terminate();
    }

    /** @deprecated */
    public Object queryElement(int e, String eq) {
	return sim.queryElement(e, eq);
    }

    public Object queryElement(int e, Query q) {
	return sim.queryElement(e, q);
    }

    /** @deprecated */
    public Vector queryPopulation(String p, String eq) {
	return sim.queryPopulation(p,eq);
    }
    public java.util.List queryPopulation(String p, Query q) {
	return sim.queryPopulation(p,q);
    }
    public int lookupElementID(String name) {
	return sim.lookupElementID(name);
    }
    public ElementState lookupElement(String name) {
	return sim.lookupElement(name);
    }

    public PopulationState lookupPopulation(String name) {
	return sim.lookupPopulation( name );
    } 
    public PopulationState getRootElement() {
	return sim.getRootElement();
    }
    public Kernel getKernel(){ 
        return sk;
    }
    
    public Object getReader() {
        return ba;
    }
    
}
