package neuroml.core;

import java.util.Vector;

/** An array of doubles.
 * Implemented using java.util.Vector
 *
 * Alternative: Just have:

public MyClass {
  public double[][] dataPoints;

}

<neuroml class="MyClass">
  <dataPoints ndims="2" >
    1 2 3 4 56 3 2 45 3 2
  </dataPoints>
</neuroml>




 */
public class DoubleArray {
    String name;

    double[] data;  
    Ref unit;
    int ndims=1;

    Vector<Double> vec = null; // Store data as a vector...

    public DoubleArray() { unit=new Ref("Unit"); }
    public DoubleArray(Ref unit) { this.unit=unit; }
    public DoubleArray(int ndims) { unit= new Ref("Unit"); this.ndims = ndims; }

    public String getName() { return name; }
    public void setName(String n) { name = n; }    

    /** Set the data in this array to d[]
     */
    public void setData(double[] d) { 
	data = d; 
    }    

    /** Set the data in this array to the vector of Double() objects
     */
    public void setData(Vector<Double> v) {
	data = new double[v.size()]; 
	for (int i=0; i<v.size(); i++) {
	    data[i] = (v.elementAt(i)).doubleValue();
	}
	vec = v;
    }
   
    /** Add an element to the array 
     */
    public void addElement( double v ) {
	if (vec == null) { vec = new Vector<Double>(); }
	vec.addElement( new Double(v) );
    }

    /** Add an element to the array 
     */
    public void addElement( Double v ) {
	if (vec == null) { vec = new Vector<Double>(); }
	vec.addElement( v );
    }

    /** Return data as a double[] */
    public double[] getData() { 
	if (vec != null) setData( vec );
	return data; 
    }
    
    /** Get length of vector
     */
    public int getSize() {
	if (vec !=null) return vec.size();
	return data.length;
    }

    /** Get number of dimensions
     */
    public int getNDims() {
	return ndims;
    }

    /** Return ref to unit
     */
    public Ref getUnitRef() { return unit; }
}
