package neuroml.model.channel;

import neuroml.core.*;

/** Wrapper class for channels that will be stored in a database,
  * similar to NeuromlPkg or ChannelPkg, but with additional descriptive
  * fields, and without other more general model fields.
  * @author David Beeman, University of Colorado, Boulder
  */
public class DBChannel extends Pkg {    

    /** Description
     */
    public String description = "";

    /** Author
     */
    public String author = "";

    /** Space separated list of keywords
     */
    public String keywords = "";

    /** A unique identifier used by the database to allow multiple channels
      * to have the same short names.  Generated from "UTC time" + "hex process
      * and thread ID's of running database program" + "@" +
      * "fully qualified domain name of host generating the entry".
      */
    public String uniqueID = "";

    /** For additional notes
      */
    public String notes = "";

    /**  The nominal resting potential of the cell for which this channel
      * was developed.  The use of this parameter is optional, and it should
      * be ignored if not initialized.  When a channel is used in cell models
      * with different resting potentials, Erest may be used to shift the
      * voltage dependence of the gating rates.  It may also be used as a
      * parameter in the equation string of an EquationHHRate object.
      */
    public DValue Erest;

    /** The channel definitions.  Normally only a single channel is stored
      * in a database entry, but the use of a Set allows a set of related
      * channel objects such as Ca channels, Ca concentration pools, and Ca
      * concentration dependent channels to be stored together.
    */
    public Set channels = new Set("Channel");
    public static String[] getClassList() { 
	String[] ret = {
	    "DBChannel",
	    "Channel",
	    "ChannelPkg",
	    "ConcenPool",
	    "CurrentSource",
	    "HHChannel",
	    "HHGate",
	    "HHCGate",
	    "HHVGate",
	    "TabulatedHHRate",
	    "TabulatedHHGate",
	    "TabulatedHHVGate",
	    "ParameterizedHHVGate",
	    "ThermodynamicHHVGate",
	    "HHRate",
	    "EquationHHRate",
	    "ParameterizedHHRate",
	    "KSChannel",
	    "KSState",
	    "KSTransition",
	    "KSTDepTransition",
	    "KSVDepTransition",
	    "KSConcDepTransition",
	    "KSRelativeTransition",
	    "Pore",
	    "PorePermeator",
	    "Solution",
	    "Specie"
	};
	return ret;
    }
}
