package neuroml.model.channel;

import neuroml.core.*;

/** An ionic concentration-dependent gate, analogous to the voltage-dependent
  * HHVGate.  It provides flexibility in the choice of what Rate classes are
  * used for forwardRate and backwardRate by casting them to the appropriate
  * subclass of HHRate.
  * @author David Beeman, University of Colorado, Boulder
  */

public class HHCGate extends HHGate {

    public HHRate forwardRate = new HHRate();
    public HHRate backwardRate= new HHRate();

    /** Time unit used with the rate variables. e.g. "sec" or "msec".
     */
    public Ref timeUnit = new Ref("Unit");

    /** Concentration unit used for the rate variables. e.g. "mole/m^3"
     */
    public Ref concentrationUnit = new Ref("Unit");

    /** Minimum ionic concentration expected to be applied to the gate
    */
    public double cmin;

    /** Maximum ionic concentration expected to be applied to the gate
    */
    public double cmax;

    /** Reference to the object that provides the source of the ionic
        concentration */
    public Ref concenSource = new Ref("ConcenPool");

/** If true, perform an instantaneous calculation of the gate activation
 * variable, setting it equal to forwardRate/(forwardRate + backwardRate).
 * The default (false) is to solve the Hodgkin-Huxley DE's.
 */
    public boolean instantCalculation = false;

/** If true, use forwardRate, backwardRate for state variables tau, minf
 *  instead of the default of using rate variables alpha, beta.
*/
    public boolean useState = false;

    public HHCGate() {
    }

    public HHCGate(String name) {
	super(name);
    }

}
