package neuroml.model.channel;

import neuroml.core.*;


/**  Parameterized Hodgkin-Huxley rates for alpha or beta.
 *   The expression is: <BR>
 *   rate = (A + BV) / (C + D exp((E + V)/F)). <BR>
 *   Note that this can specify the 
 *   same expression with different parameter sets: fo example one coulde 
 *   divide through by D. The extra paremeters allow it to represent 
 *   a range of cases, as, for example where D=0.
 */
public class ParameterizedHHRate extends HHRate {

    public double A, B, C, D, E, F;
    public Ref timeUnit = new Ref("Unit");
    public Ref voltageUnit = new Ref("Unit");


    public ParameterizedHHRate() {
    }

    public ParameterizedHHRate(String name) {
	super(name);
    }

}
