package neuroml.model.channel;

import neuroml.core.*;

/** Linear thermodynamic gating model, where the effective forward rate is<br>
 *  forwardRate = alpha0 exp(-gamma gatingCharge (F/RT) V) <BR>
 *  reverseRate = beta0 exp((1-gamma) gatingCharge (F/RT) V) <BR>
 *  That is, alpha0 is the forward rate at sero membrane potential.
 *  This is equivalent to the Borg-Graham expressions:  <BR>
 *  rates = A exp([-gamma : (1-gamma)] q (V - Vh) (F/RT)) <BR>
 *  
 *  User interface tools may allow rates to be expressed either way, but 
 *  the underlying model storage is with the gatingCharge and gamma, not 
 *  Vhalf. 
 */ 
public class ThermodynamicHHVGate extends HHGate {
     
    
    public DValue forwardRate = new DValue();
    public DValue backwardRate = new DValue();
    public DValue gatingCharge = new DValue();
    public DValue gamma = new DValue();


    public ThermodynamicHHVGate() {
    }

    public ThermodynamicHHVGate(String name) {
	super(name);
    }

}
