package neuroml.sim.run;
 
/** A connection request is the message which a source cell sends to a dest cell
 * it wants to connect to. By default it contains the source entity
 * ID and port no, as well as the DestMethod.
 */
public class ConnectionRequest implements java.io.Serializable {

    /** ID of source cell 
     */
    protected int srcID;

    /** Source output no 
     */
    protected short srcPortID;

    /** Method to run @ destination end 
     */
    protected DestMethod destMethod;

    /** Placeholder for extra info about source entity which 
     * destination needs to know in order to build connection
     */
    protected Object srcData = null;

    /** Construct a connection request
     * Contains all information needed at destination end to
     * construct the connection.  At a minimum this is the source 
     * ID and output port ID.
     */
 //   public static int nbconnSent=0;
    public ConnectionRequest(int srcID, short srcPortID, DestMethod destMethod ) {
	this.srcID = srcID;
	this.srcPortID = srcPortID;
	this.destMethod = destMethod;
//        nbconnSent++;
    }

    /** Construct a connection request with extra data re source 
     * (e.g. position)
     * Contains all information needed at destination end to
     * construct the connection.  At a minimum this is the source 
     * ID and output port ID.
     */
    public ConnectionRequest( int srcID, short srcPortID, DestMethod destMethod, Object srcData ) {
	this.srcID = srcID;
	this.srcPortID = srcPortID;
	this.destMethod = destMethod;
	this.srcData = srcData;
   //     nbconnSent++;
    }    

    public int getSrcID() { return srcID; }
    public short getSrcPortID() { return srcPortID; }
    public DestMethod getDestMethod() { return destMethod; }
    public Object getSrcData() { return srcData; }

    /** Set some extra data about the source that the target needs to know to build 
     * the connection, e.g. the position
     */
    public void setSrcData(Object d) { srcData = d; }
}
