package neuroml.sim.run;  

/** A simple query whch just has a name and parameters
 * Sample use fom script:
 * <tt>print sim.query( 123, new NamedQuery("getname",null) );</tt>
 * Note that it is the Query which is responsible for extracting the information from
 * the object.
 */
public class NamedQuery implements Query {
    String name;
    Object parameters;

    /** Construct a query with a name and parameters
     * This is an alternative to subclassing Query
     * @param name the name of this query
     * @param p optional query parameters
     */
    public NamedQuery(String name, Object p) { 
	this.name = name; 
	this.parameters = parameters; 
    }

    /** Runs this query on the given object.
     * Implementations of this perform any required query on an object.
     * @param o the object to query
     * @return the result of the query
     */
    public Object query(Object o) {
	if (o instanceof NamedQueryable) {
	    return ((NamedQueryable)o).queryByName(name, parameters);
	}
	return new Integer(0);
    }

    /** Returns the name of this query
     */
    public String getName() { return name; }

    /** Returns the parameters of this query
     */
    public Object getParameters() { return parameters; }
}
