package neuroml.sim.state;

import neuroml.model.network.*;
import java.util.List;

/** Interface to access the state of a population instance
 */
public interface PopulationState extends ElementState {
   
    /** Returns number of elements
     */
    int getNumElements();

    /** Returns index of a given element ID
     * @param eid handle of element
     * @return index of element in population if present, or -1 otherwise.
     */
    int getIndex( int elementID );

    /** Returns elementID of a given index
     * @param index index in population
     * @return elementID of given index
     */
    int getElementID( int index );
    
    /** Returns whether a given element is a member
     * @param eid handle of element
     * @return true if element is a member of this population
     */
    boolean isMember( int elementID  );

    /** Returns a list of all members' ids
     */
    java.util.List getElementIDs();

    /** Returns a list of all members' names
     */
    java.util.List getElementNames();

    /** Adds a member to this population
     * @param elementID handle of element
     * @param name name of element
     */
    void addElement( int elementID, String name );
}
