package neuroml.util;

import neuroml.core.*;
import neuroml.*;
import neuroml.model.channel.*;
import neuroml.model.cell.*;
import neuroml.model.network.*;
 
import java.lang.reflect.*;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Collection;
import java.lang.Integer;
import java.net.*;

import neuroml.core.StringArray;//JAG
import neuroml.util.RefInterfaces;
import java.util.*;
 
/**
 * A neuroml namespace.
 * Used for references / dererefences.
 * <p>
 * 
 *
 * A separate namespace is used for each xml document.
 * <pre>
 * "document1"
 *    "Networks"
 *       "nw1"
 *       "nw2"
 * "document2"
 *    "Cells"
 *       "cell1"
 *    "Networks"
 *       "nw1" // distinct from document1.nw1
 *       "nw2"
 * "document3"
 *    "Channels"
 *       "channel1"
 *       "channel2"
 * </pre>
 *
 * @author Paul Rogister and Fred Howell 
 *
 * @version 1.0, 25 July 2001  
 * @version 1.1, 13 August 2001  -- added "derivedFrom" method
 *                 
 */
public class Namespace {

    String name;

    private boolean debug = false;

 
    /** The references are stored as a hashtable{type} of hashtables{refname},
     * "Network"
     *    "network123" --> Network object
     *    "network456" --> Network object
     *    "network876" --> Network object
     * "Cell"
     *    "cellpkg.cell2" --> Cell object
     *    "cellpkg.cell6" --> Cell object
     *    "cellpkg.cell9" --> Cell object
     */
    Hashtable<String,Hashtable<String,Object>> references = new Hashtable<String,Hashtable<String,Object>>();

    /** The classes available to instantiate
     * "neuroml.core.Module"
     *    --> "neuroml.core.ProjectionModule"
     *    --> "neuroml.core.PopulationModule"
     *    --> "mycode.MyModule"
     * "neuroml.model.network.PopulationStructure"
     *    --> "neuroml.model.network.IndexedStructure"
     * Hashtable{className} of Vector{String}
     * <p>
     * Needs to be updated by the ModuleLoader
     * And by any applications which add classes which 
     * need to be constructable by the editor.
     */
    Hashtable<String,Vector<String>> derivedFrom = new Hashtable<String,Vector<String>>();

    /**
     * list of jar file to be used in the classloader classpath
     */
    URL[] jarUrls = new URL[0];

 //JAG needed for the module identification
    Vector<Class> ArrayClasses = new Vector<Class>(1,1);
    Vector<Class> ArrayInterfaces = new Vector<Class>(1,1);
    Vector<Object> ArrayModuleRef = new Vector<Object>(1,1);
    Vector<String> ArrayNameModule = new Vector<String>(1,1);    
    
    
    /** Add a parent/child relationship for a class 
     */
    public void addClass(String parent, String className) {
	Vector<String> v = ( derivedFrom.get(parent) );
	if (v==null){ 
            v = new Vector<String>(); 
            derivedFrom.put(parent, v); 
        }
	if (!v.contains(className)) v.addElement(className);
    }

    /** Make a recusrive list of all classes derived from parent */
    void addDerivedFrom( Vector<String> ret, String parent ) {
	Vector<String> v = (derivedFrom.get(parent));
	if (v!=null) {
	    for (int i=0; i<v.size(); i++) {
		String child = v.elementAt(i);
		ret.addElement(child);
		addDerivedFrom( ret, child );
	    }
	}
    }
    
    /** Return a list of classes derived from parent 
     * e.g. getDerivedFrom( "neuroml.model.network.PopulationStructure")
     * might return a vector with "IndexedStructure", "mymodule.MyStructure",
     * etc.
     * @return a vector of Strings with the full class names of any classes 
     *  derived from the given class.
     * @see initialiseDerivedFrom
     */
    public Vector<String> getDerivedFrom( String parent ) {
	Vector<String> ret = new Vector<String>();
	addDerivedFrom(ret,parent);
	return ret;
    }

    /** Add all parents of a class to derivedFrom table... */
    //JAG check interfaces
    public void addParents( Class c ) {
	if (c==null) return;

        Class Interfaces[]=c.getInterfaces();
        int i,j=Interfaces.length;
        for(i=0;i<j;i++)
            {
                String string = Interfaces[i].getName();
                addClass(string, c.getName());
		addParents(Interfaces[i]);     
            }
        
        Class p = c.getSuperclass();
        if (p==null) return;
	String pname = p.getName();
	if (pname.equals("java.lang.Object")) {
	    // Stop here...
	} else {
	    addClass(pname, c.getName());
	    addParents( p );
	}	
    }

   /** Add url of jar file to classloader classpath*/
    public void addJarUrl( URL jarurl ) {
	// if url not already in (if not, privilege exception..
	if (!alreayIn(jarurl,jarUrls)){
	    int oldsize = jarUrls.length;
	    jarUrls = new URL[oldsize+1];
	    jarUrls[oldsize] = jarurl;
	}
    }

    /** Add url of jar file to classloader classpath*/
    public boolean alreayIn( URL jarurl , URL[] jarurlarray ) {
	boolean in = false;
	for (int i=0; i<jarurlarray.length && !in ;i++)
	    in = (jarurlarray[i].toString().equalsIgnoreCase(jarurl.toString()));
	return in;
    }
    

  /** Add all parents of a class to derivedFrom table... */
    public void addJarUrl( String jarurlname ) {
	int oldsize = jarUrls.length;
	jarUrls = new URL[oldsize+1];
	URL jarurl = null;
	try {
	     jarurl = new URL(jarurlname);
	} catch (MalformedURLException eurl){
	    //System.out.println("Malformed URL "+eurl);
	}
	if (jarurl != null)
	    jarUrls[oldsize] = jarurl;
    }

  /** Try to make a class */
    public Class createClass(String className) {
        Class classDefinition =null;
	try {
	    classDefinition = Class.forName(className);
	    if (debug) System.out.println(" Namespace createClass "+className);
        } catch (Exception e) {
	    System.out.println(" Exception Namespace createClass "+className+" "+e);
        } 
        return classDefinition;
    }

    /** Initialise the derivedFrom with classes in neuroml packages
     *	e.g.   initialiseDerivedFrom(NeuromlPkg.getClassList()); 
     */
    public void initialiseDerivedFrom( String[] classes ) {
	for (int i=0; i<classes.length; i++) {
	    Class c = null;
	    try {
		c = NeuromlSys.createClass(classes[i]);
               // NEED TO BE MOVED SO THAT JAR ENTRIES ARE ALSO TAKEN INTO ACCOUNT DURING EDITING WHEN THE .JAR FILES AREN'T IN THE CLASS PATH
               // addClassToModules(c);//JAG
		if (debug) System.out.println("initialiseDerivedFrom creates class "+classes[i]);

	    } catch (Exception e) {
		//System.out.println("initialiseDerivedFrom exception : "+e);

	    } catch (java.lang.UnsatisfiedLinkError er) {
		//System.out.println("Ignoring UnsatisfiedLinkError");
	    } catch (java.lang.NoClassDefFoundError ef) {
		//System.out.println("Ignoring NoClassDefFoundError");
	    } 
	    if (c==null)
	    	c = NeuromlSys.getClassFromJar(classes[i]);
	    if (c!=null){
                addClassToModules(c);//JAG
		//System.out.println("initialise "+classes[i]+" addParents "+c);
		addParents( c );
	    }
	    //System.out.println("initialise "+classes[i]+" addParents "+c);
			
	}	
         getListModuleRef(classes);
    }

    /** Print registered classes 
     */
    public void printDerivedFrom() {
	for (Enumeration enum_ = derivedFrom.keys() ; enum_.hasMoreElements() ;) {
	    String key = (String)enum_.nextElement();
	    Vector<String> v = (derivedFrom.get(key));
	    System.out.println("  "+key+" --> ");
	    if (v!=null) {
		for (int i=0; i<v.size(); i++) {
		    System.out.println("    "+v.elementAt(i));
		}
	    }
	}

    }

    /** A hashtable of lists of list info:-
     * "Network"
     *    networkpkg.networks
     * "Cell"
     *    cellpkg.cells
     * "Module"
     *    cellpkg.modules
     *    networkpkg.modules
     */
    public Hashtable<String,Vector<ListInfo>> lists = new Hashtable<String,Vector<ListInfo>>();

    /** Add details about a list (where referenced objects are stored)
     * Currently added by "addReferences" call.
     * Should also be updated by GUI
     */
    public void addListInfo( String type, ListInfo li ) {
	Vector<ListInfo> v = lists.get(type);
	if (v==null) {
	    v = new Vector<ListInfo>();
	    lists.put(type, v);
	}
	v.addElement( li );
    }

    /** Get a list of lists for a given type
     * @return a Vector of ListInfo
     * @see ListInfo
     */
    public Vector<ListInfo> getListInfo(String type) {
	Vector<ListInfo> v = lists.get(type);
	return v;
    }


    /** Make a new namespace with given name
     */
    public Namespace(String name){
	// Put units into them all... probably a mistake
	this.name = name;
        //BAD CAST !!!
	Hashtable unitRefs =Unit.getStandardUnits();
	references.put("Unit",unitRefs);

	// Initialise derivedFrom with all classes in neuroml
	initialiseDerivedFrom(NeuromlPkg.getClassList()); 
    }
    
    public String getName() { return name; }


    /** gives the shortclass name, the last name after the last '.'
     *
     */
    public static String getShortClassName(String className){
	int index = className.lastIndexOf('.');
	if (index!=-1)
	    return className.substring(index+1);
	else 
	    return className;
    }

    /**
     * Add all references in a package.
     * NetworkPkg { 
     *    Set populations { "pop1", "pop2" };
     *    Set projections { "pop6", "proj7" };
     * } 
     * ... adds "populations.pop1", "populations.pop2"
     */
    public void addReferences(Object pkg) {
        Object a=pkg; //store the package
        
	if (pkg != null) {
	    addReferences("", "",pkg);
	}  
        RecurrentUpdateRef(a);
    }
    
    private void RecurrentUpdateRef(Object o){
        Field[] flds = o.getClass().getFields();
        for(int i=0; i<flds.length; i++) {
            try {
            	Object fob = flds[i].get(o);
                if (fob == null) {
                } else if (fob instanceof String ) {
                } else if (fob instanceof Integer || fob instanceof Short) {
                } else if (fob instanceof Double ) {
                } else if (fob instanceof DValue ) {
                } else if (fob instanceof Boolean ) {                           
                } else if (fob instanceof URL ) {
                } else if (fob instanceof neuroml.core.Ref ) {
                    UpdateRefered("",((Ref)fob).getTargetName(), ((Ref)fob));
                } else if (fob instanceof neuroml.core.Set ) {
                    int k=((neuroml.core.Set)fob).size();
                    for(int j=0;j<k;j++)
                        RecurrentUpdateRef(((neuroml.core.Set)fob).elementAt(j));
                } else {
                      RecurrentUpdateRef(fob);
                }//for
            }
            catch(Exception e){
                
            }
        }
    }
    /** realPath is fullPath,
     * path is path since last Pkg
     * Path includes "."
     */

    private void RecurrentUpdateRefNetwork(Object o,String OldName,String NewName){
        Field[] flds = o.getClass().getFields();
        for(int i=0; i<flds.length; i++) {
            try {
            	Object fob = flds[i].get(o);
                if (fob == null) {
                } else if (fob instanceof String ) {
                } else if (fob instanceof Integer || fob instanceof Short) {
                } else if (fob instanceof Double ) {
                } else if (fob instanceof DValue ) {
                } else if (fob instanceof Boolean ) {                           
                } else if (fob instanceof URL ) {
                } else if (fob instanceof neuroml.core.Ref ) {
                    String sample=((Ref)fob).getTargetName();
                    if(sample.startsWith(OldName)){
                        sample=sample.substring(OldName.length());
                        ((Ref)fob).setTargetName(NewName+sample);
                    }
                } else if (fob instanceof neuroml.core.Set ) {
                    int k=((neuroml.core.Set)fob).size();
                    for(int j=0;j<k;j++)
                        RecurrentUpdateRefNetwork(((neuroml.core.Set)fob).elementAt(j),OldName,NewName);
                } else {
                      RecurrentUpdateRefNetwork(fob,OldName,NewName);
                }//for
            }
            catch(Exception e){
                
            }
        }
    }    
    
    public void addReferences(String realPath, String path, Object o) {
	// check fields which are Set or List
	Class c =  o.getClass();
	Field[] flds = c.getFields();
	for (int i=0; i<flds.length; i++) {
	    try {
		Object fob   = flds[i].get(o);
		String fname = flds[i].getName();

		if (fob instanceof java.util.Collection) { 
		   // Vector v;
		    String eltype="java.lang.Object";
		    Iterator it = ((Collection)fob).iterator();
		    if (fob instanceof ContentRestricted) {
			eltype = ((ContentRestricted)fob).getContentClassName();
		    }   

		    /** Add list info for this list...*/
		    addListInfo(eltype, new ListInfo( o, realPath, fname ) );

		    while (it.hasNext()) {
			Object el = it.next();
			if (el instanceof Nameable) {	// Only referencing Nameables... not strictly necessary	
			    String elname = ((Nameable)el).getName();
			    // Actually add a reference
			    addRef(path+fname+"."+elname, el);
			    addReferences(realPath+fname+"."+elname+".",
					  path+fname+"."+elname+".", el);
			    if (debug) {
				System.out.println("addReference "+realPath+fname+"."+elname+"."+" ,  "+
						   path+fname+"."+elname+".");
			    }
			    // circular references a problem...
			}
		    }
		} else if (fob instanceof Nameable)  {
		    // Recurse downwards, only prepend path if parent not pkg
		    // This means references have to know where they are in the tree,
		    // i.e. require a "path"...
		    if (! (o instanceof Pkg)) {
			addReferences(realPath+fname+".",path+fname+".", fob);
		    } else {
			//			addReferences(realPath+fname+".","", fob);
			addReferences(realPath+fname+".",path+fname+".", fob);
		    }
		}
	    } catch (Exception e) {
		System.out.println("Namespace.addreferences : "+e);
	    }
	}

	
    }


    /** Rename a reference
     * e.g. <tt>a.b.fred a.b.jim</tt>
     * also rename any sub references.
     *  a.b.fred.jane  -> a.b.jim.jane
     * This is slow! Maybe hashtables aren't the best solution?
     * Just a tree???
     * A treemap???
     */
    public void rename(String oldkey, String newkey) {
	if (debug) System.out.println("Namespace: Renaming "+oldkey+" as "+newkey);
       
	int okl = oldkey.length();
	for (Enumeration t = references.keys(); t.hasMoreElements();) {
	    Hashtable<String,Object> objs = references.get((String)t.nextElement());
	    Hashtable<String,Object> movedobjs = new Hashtable<String,Object>();
	    for (Enumeration o = objs.keys(); o.hasMoreElements();) {
		String refname = (String)o.nextElement();
		if (refname.compareTo(oldkey)==0||refname.startsWith(oldkey+".")) {//startsWith
		    // We have a replace to do!!!
	//	    Object theob = objs.remove(  refname ); JA
                    Object theob = objs.get(  refname );
		    String newstring = newkey+refname.substring(okl);
		    movedobjs.put( newstring, theob );
		    if (debug) System.out.println("Renaming "+refname+" as "+newstring);
		}
	    }
	    objs.putAll(movedobjs);
	}
        okl = oldkey.length();
        this.UpdateRefered(oldkey,newkey);
	for (Enumeration t = references.keys(); t.hasMoreElements();) {
	    Hashtable<String,Object> objs = references.get((String)t.nextElement());
	
	    for (Enumeration o = objs.keys(); o.hasMoreElements();) {
		String refname = (String)o.nextElement();
		if (refname.compareTo(oldkey)==0||refname.startsWith(oldkey+".")) { //startsWith
		    // We have a replace to do!!!
		    Object theob = objs.remove(  refname );
		}
	    }
	}
        
    }

    /**
     * Add a reference to this object.
     * (build the key before using this method)
     */
    public void addRef(String key, Object obj) {
	if (debug) System.out.println(getName()+" adding reference to "+key);
	if (key==null)  
	    return;
	Class sc = obj.getClass();	
       	boolean doLoop = true;
	while (doLoop){
	    if (sc!=null) {		
		Package p = sc.getPackage();
		if (p.getName().startsWith("neuroml")) {
		    String className =  getShortClassName(sc.getName());		    
		    if (references.containsKey(className)){
			Hashtable<String,Object> typeReference = references.get(className);
			if (typeReference.containsKey(key)) {			   
			} else {
			    typeReference.put(key,obj); 	
			}
		    } else {
			Hashtable<String,Object> typeReference = new Hashtable<String,Object>();
			typeReference.put(key,obj);
			references.put(className,typeReference);			
		    }
		    sc = sc.getSuperclass();
		} 
		else doLoop = false;
	    }
	    else doLoop = false;
	}
        Vector<Ref> v=refered.get(key);
        if(v!=null){
            refered.remove(key);
        }
    }

    /**
     * Remove a reference in the hashtable used by deRef
     * if element without name cannot be added, this has to be checked
     * by the key building method called before any call to deregisterForRef
     */
    public void removeRef(String key, Object obj) {
	Class sc = null;
	sc = obj.getClass();
	boolean doLoop = true;

	//System.out.println("removeRef sc "+sc);

	while(doLoop){
	    if (sc!=null) {
	    	//System.out.println("removeRef sc "+sc);

		Package p = sc.getPackage();
		//System.out.println("removeRef p "+p);
		if (p!=null){ //can be null when JarClassLoader is used
		    if (p.getName().startsWith("neuroml")) {// belongs to neuroml package so we carry on 
			String className =  getShortClassName(sc.getName());		
			// use it as a key in the references hashtable
			if (references.containsKey(className)){
			    // if the type key exist, try to remove the object from its type hashtable
			    Hashtable<String,Object> typeReference = references.get(className);
			    // remove
			    typeReference.remove(key);
			} else { // no such type hashtable, so nothing to remove
			    //System.out.println(" deregisterForRef : no such referenced class : "+className);
			}
			sc = sc.getSuperclass();
		    }    
		    else doLoop = false;
		}
		else doLoop = false;
	    } 
	    else doLoop = false;
	}
    }

    /**
     * lookupRef : lookup a key in a type hashtable
     */
    private Object lookupRef(String key, Hashtable<String,Object> typeRefs) {
	Object target = null;
        if(key==null) return null;
	if (typeRefs.containsKey(key)){
	    target = typeRefs.get(key);
	} else {
	    boolean foundOne = false;
	    for (Enumeration enum_ = typeRefs.keys() ; enum_.hasMoreElements() ;) {
		String aKey = (String)enum_.nextElement();
		if (aKey.endsWith(key)){
		    if (typeRefs.containsKey(aKey)){
			target = typeRefs.get(aKey);
			foundOne = true;	
		    }
		}
		if (foundOne) break;
	    }
	}
	return target;
    }

    private String lookupRefKey(String key, Hashtable<String,Object> typeRefs) {
	String target = null;
        if(key==null) return null;
	if (typeRefs.containsKey(key)){
	    for (Enumeration enum_ = typeRefs.keys() ; enum_.hasMoreElements() ;) {
		String aKey = (String)enum_.nextElement();
                if(aKey.compareTo(key)==0){
                    target=key;
                    break;
                }
            }

        }
	return target;
    }
    /**
     * deRef : return the object targeted by a Ref
     */
    public Object deRef(Reference ref) {
	Object target = null;
	String type = "Object"; 
	if (ref instanceof ContentRestricted) { type = ((ContentRestricted)ref).getContentClassName(); }
	Hashtable<String,Object> typeReference = references.get(type);
	if (typeReference==null) {
	} else {
	    String name = ref.getTargetName();
	    target = lookupRef(name,typeReference);	    
	}
	return target;	
    }
     public void deRef(Reference ref,String key) {
	String target = null;
	String type = "Object"; 
	if (ref instanceof ContentRestricted) { type = ((ContentRestricted)ref).getContentClassName(); }
        if(type==null)
            return;
	Hashtable<String,Object> typeReference = references.get(type);
	if (typeReference==null) {
	} else {
	    String name = ref.getTargetName();
	    target = this.lookupRefKey(name,typeReference);	
            ref.setTargetName(target);
	}
	//return target;	
    }
    
    public Hashtable<String,Hashtable<String,Object>> getReferences(){
	return references;
    }
    
        /*
     *  JAG
     *  return a Vector containing classes inheriting from both SuperClass and all elements of Interfaces
     */
    
    public Vector<Class> getModuleClasses(String NameOfModule)
    {      
        Vector<Class> InModule=new Vector<Class>(1,1);
        Class O;
        int i,j=ArrayClasses.size(),k=-1;
        boolean MayBe;
        if(!ArrayNameModule.contains(NameOfModule)) 
            return null;
        RefInterfaces theStuff=(RefInterfaces)ArrayModuleRef.elementAt(ArrayNameModule.indexOf(NameOfModule));
        String SuperClass=theStuff.SuperClass;
        String[] Interfaces=theStuff.Interfaces.getData();
        
        int l=Interfaces!=null?Interfaces.length:0;
        Class _Super=SuperClass!=null?NeuromlSys.classLoader.getLoadedClass(SuperClass):null;
        Class [] _Inter=new Class[l];
               
        for(i=0;i<l;i++)
            try{
                k++;
                _Inter[k]=NeuromlSys.classLoader.getLoadedClass(Interfaces[k]);
            } catch (Exception exception) {}    
        for(i=0;i<j;i++)
        {
            try {
		O = ArrayClasses.elementAt(i);
                MayBe=true;               
                if(_Super!=null) MayBe=MayBe&_Super.isAssignableFrom(O);
                for(k=0;MayBe&&k<l;k++)
                    MayBe=(_Inter[k]!=null)?_Inter[k].isAssignableFrom(O):false;
                if(MayBe)
                    InModule.add(O);
	    } catch (Exception exception) {}    
        }
        return InModule;       
    }
    
    protected void addClassToModules(Class myClass)
    {
        if(myClass==null) return;
        if(myClass.isInterface()){
            if(!ArrayInterfaces.contains(myClass)) ArrayInterfaces.add(myClass);
        }   
        else{
            if(!ArrayClasses.contains(myClass)) ArrayClasses.add(myClass);
        }
    }
    
    protected void getListModuleRef(String[] strings)
    {
	for (int i = 0; i < strings.length; i++) {
	    Class var_class = null;
	    try {
		var_class = NeuromlSys.createClass(strings[i]);
                if(var_class!=null)
                {
                    if(neuroml.core.ModuleRef.class.isAssignableFrom(var_class))
                    {
                        //ModuleRef not required apparently, just attributes to module ref...
                        /*
                        String muche=var_class.getName();
                        if(!ArrayNameModule.contains(muche)) 
                        {
                            ArrayNameModule.addElement(muche);
                            RefInterfaces thatRef=new RefInterfaces(muche);
                            ArrayModuleRef.addElement(thatRef);
                        }
                         */
                    }
                    else
                    {
                        Field[] fields=var_class.getFields();
                        if(fields!=null)
                        {
                            for(int j=0;j<fields.length;j++)
                            {                               
                                if(neuroml.core.ModuleRef.class.isAssignableFrom(fields[j].getType()))
                                {
                                    String str=var_class.getName();
                                    String str2=fields[j].getName();
                                    String muche=str+"."+str2;
                                    
                                    if(!ArrayNameModule.contains(muche)){ 
                                        ArrayNameModule.addElement(muche);
                                        RefInterfaces thatRef=new RefInterfaces(muche);
                                        ArrayModuleRef.addElement(thatRef);
                                    }
                                }
                            }
                        }
                    }
                }
	    } catch (Exception exception) {
		/* empty */
	    } catch (UnsatisfiedLinkError unsatisfiedlinkerror) {
		/* empty */
	    } catch (NoClassDefFoundError noclassdeffounderror) {
		/* empty */
	    }//Exception
        }//for i     
    }
    public Vector<Object> getModuleRef()
    {
        return ArrayModuleRef;
    }
    
    public void setModuleRef(Vector<Object> myVec)
    {
        //ArrayModuleRef=myVec;
        for(int i=0;i<myVec.size();i++)
        {
            try{
                RefInterfaces A=(RefInterfaces)myVec.elementAt(i);
                if(ArrayNameModule.contains(A.getName()))
                {
                    RefInterfaces B=new RefInterfaces(A.getName());
                    B.editorInfo=A.editorInfo;
                    //(RefInterfaces)ArrayModuleRef.elementAt(ArrayNameModule.indexOf(A.getName()));
                    B.SuperClass=A.SuperClass;
                    B.Interfaces=A.Interfaces;                    
                    ArrayModuleRef.setElementAt(B,ArrayNameModule.indexOf(A.getName()));
                }
                else
                   System.out.println("Unrecognized ModuleRef "+A.getName());
            }
            catch(Exception E0){
            }
        }
    }
    
     public Vector<Class> getArrayClasses()
    {
        return ArrayClasses;
    }
     
     public Vector<Class> getArrayInterfaces()
    {
        return ArrayInterfaces;
    }
     
         public String[] getListClasses()
    {
        Vector<String> v=new Vector<String>(1,1);
        String[] thing=new String[ArrayClasses.size()];
        for(int i=0;i<thing.length;i++)
            v.addElement((ArrayClasses.elementAt(i)).getName());
        Collections.sort(v);
        for(int i=0;i<thing.length;i++) 
             thing[i]=v.elementAt(i);
        return thing;
    }
    public String[] getListInterfaces()
    {
        Vector<String> v=new Vector<String>(1,1);
        String[] thing=new String[ArrayInterfaces.size()];
        for(int i=0;i<thing.length;i++)
            v.addElement((ArrayInterfaces.elementAt(i)).getName());
        Collections.sort(v);
         for(int i=0;i<thing.length;i++) 
             thing[i]=v.elementAt(i);
        return thing;
    }
    public void UpdateRefInterfaces(String Much, RefInterfaces Truc)
    {
        if(ArrayNameModule.contains(Much)) 
            ArrayModuleRef.setElementAt(Truc, ArrayNameModule.indexOf(Much));
    }
    
    Hashtable<String,Vector<Ref>> refered = new Hashtable<String,Vector<Ref>>();
    
    public void AddTreeRefered(String NewName, Ref myReference){
        if(NewName!=null&&NewName.compareTo("")!=0){
            Hashtable<Integer,String> new_=getReferedTreeRef((TreeRef)myReference,NewName);
            Enumeration enum_;
            enum_=new_.keys();
            while(enum_.hasMoreElements())
            {
                 Integer k = (Integer)enum_.nextElement();
                 String k2=new_.get(k);
                 Vector<Ref> v=new Vector<Ref>();
                 if(!refered.containsKey(k2)){
                     refered.put(k2, v);
                 }
                 else 
                    v=refered.get(k2);
                  if(!v.contains(myReference))
                     v.add(myReference);
            }                
        }
    }
   /* protected String UpdateNetworkRefering(String OldName){       
        String CurrentRoot=OldName;
        String NetworkRoot=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+".";//models
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);
        NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+".";//pkg1
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);
        NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+".";//networks
        String PkgNetwork=NetworkRoot;
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);        
        NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'));//myNetworkName
        try{
            Network n=(Network)(this.lookupRef(NetworkRoot,(Hashtable) (this.references.get("Network"))));
            return NetworkRoot;//.substring(NetworkRoot.lastIndexOf('.')+1)
        }
        catch(Exception e){
            return null;
        }
    }*/
    
    public void UpdateRefered(String OldName,String NewName, Ref myReference){
   //     if(OldName!=null && NewName!=null&& OldName.compareTo(NewName)==0)//
   //         return;
        if(myReference instanceof TreeRef){
             Enumeration enum_;
            if(OldName!=null&&OldName.compareTo("")!=0){
                Hashtable<Integer,String> old=getReferedTreeRef((TreeRef)myReference,OldName);
                enum_=old.keys();
                while(enum_.hasMoreElements())
                {
                    Integer k = (Integer)(enum_.nextElement());
                    String k2=old.get(k);
                    Vector<Ref> v=refered.get(k2);
                    if(v.contains(myReference))
                        v.remove(myReference);
                    if(v.size()==0) {
                        refered.remove(k2);
                        
                    }
                }
                //special treatment for Network, we don't bother with packages
              /*  String str=UpdateNetworkRefering(OldName);
                if(str!=null){
                    if(refered.containsKey(str)){
                       Vector v=(Vector)refered.get(str);
                       v.remove(myReference);
                       if(v.size()==0) 
                            refered.remove(str);
                    }                    
                } */               
            }
            if(NewName!=null&&NewName.compareTo("")!=0){
                    Hashtable<Integer,String> new_=getReferedTreeRef((TreeRef)myReference,NewName);
                    enum_=new_.keys();
                    while(enum_.hasMoreElements())
                    {
                        Integer k = (Integer)enum_.nextElement();
                        String k2=new_.get(k);
                        Vector<Ref> v=new Vector<Ref>();
                        if(!refered.containsKey(k2)){
                            refered.put(k2, v);
                        }
                        else 
                            v=refered.get(k2);
                        if(!v.contains(myReference))
                            v.add(myReference);
                    } 
                    //special treatment for Network, we don't bother with packages
                   /* String str=UpdateNetworkRefering(NewName);
                    if(str!=null){
                        Vector v=new Vector();
                        if(!refered.containsKey(str)){
                              refered.put(str, v);
                        }
                        else 
                            v=(Vector)refered.get(str);
                        if(!v.contains(myReference))
                            v.add(myReference);
                    }*/
             }

        }
        else{
            if(OldName!=null){
                if(refered.containsKey(OldName)){
                    Vector<Ref> v=refered.get(OldName);
                    v.remove(myReference);
                    if(v.size()==0) 
                        refered.remove(OldName);
                }
            }
            if(NewName!=null){
                Vector<Ref> v=new Vector<Ref>();
                if(!refered.containsKey(NewName)){
                    refered.put(NewName, v);
                }
                else 
                    v=refered.get(NewName);
                if(!v.contains(myReference))
                    v.add(myReference);
            }
        }
        
    }
    private void UpdateRefered(String OldName,String NewName){
        Vector<Ref> v=refered.get(OldName);
        String k;
        String k2;
        String BranNewName=null,RealOldName=null;
        String TreeRefOldName=OldName.substring(OldName.lastIndexOf('.')+1);
        String TreeRefNewName=NewName.substring(NewName.lastIndexOf('.')+1);        

        Network net=(Network)(lookupRef(OldName, (this.references.get("Network"))));
        if(net!=null){
                RecurrentUpdateRefNetwork(net,OldName+".",NewName+".");  
                Enumeration enum_1=refered.keys();
                int OldNameLength=OldName.length();
                while(enum_1.hasMoreElements())
                {
                    String str = (String)enum_1.nextElement();
                    Vector<Ref> v1=refered.get(str);  
                    if(str.startsWith(OldName+"."))
                    {
                        refered.put(NewName+str.substring(OldNameLength), v1);
                        refered.remove(str);
                    }
                }
        }
        //All objects may not be refered !!!
        if(v==null){
            return;
        }
        int j=v.size();
        for(int i=0;i<j;i++){
            Ref myRef= (v.elementAt(0));
            if(myRef instanceof TreeRef){
                BranNewName=new String();
                RealOldName=myRef.getTargetName();
                //check if it is a network
                
 /*               if(RealOldName.startsWith(OldName)){
                    myRef.setTargetName(NewName+RealOldName.substring(OldName.length()));
                }
                else{*/
                    Hashtable<Integer,String> order=getReferedTreeRef((TreeRef)myRef,myRef.getTargetName());
                    Enumeration enum_=order.keys();
                    Integer k_new=null,k_old=null;
                    int diff;
                    Vector<Integer> myVector=new Vector<Integer>();
                    while(enum_.hasMoreElements()){
                         k_new = (Integer)enum_.nextElement();
                         myVector.add(k_new);
                    }
                    Collections.sort(myVector);
                    for(int x=0;x<myVector.size();x++)
                    {
                        k_new = myVector.get(x); 
                         if(k_old==null)
                             diff=k_new.intValue();
                         else
                             diff=k_new.intValue()-k_old.intValue();
                         for(int m=0;m<diff-1;m++){
                            BranNewName+=RealOldName.substring(0,RealOldName.indexOf('.'))+".";//models
                            RealOldName=RealOldName.substring(RealOldName.indexOf('.')+1);
                         }
                         int n=RealOldName.indexOf('.');
                         if(OldName.compareTo(order.get(k_new))==0){
                            BranNewName+=TreeRefNewName;
                            if(n>0) {
                                BranNewName+=".";
                                RealOldName=RealOldName.substring(n+1);
                            }
                        }
                        else{
                            if(n>0){ 
                                BranNewName+=RealOldName.substring(0,n+1);//models
                                RealOldName=RealOldName.substring(n+1);
                            }
                            else
                            {
                                BranNewName+=RealOldName;
                                RealOldName="";
                            }
                        }
                     
                        k_old=k_new;
                    }
                    myRef.setTargetName(BranNewName);
              //  }//else
                
            }
            else{
               myRef.setTargetName(NewName);
            }
        }
    }
    //Return a vector of the name of nodes and leaves of myReference refered by the Name 
    //Note: not sure it is the most computationnaly efficient at all, but there shouldn't be porblems for networks containing a small number of populations.
    public Hashtable<Integer,String> getReferedTreeRef(TreeRef myReference, String Name){
        boolean finished=false;
        Vector<String> leaves   = myReference.getLeaves();
        Vector<String> nodes    = myReference.getNodes();
        Vector<Object> Elements = myReference.getElements();
        Enumeration enum_;
        Hashtable<Integer,String> HashRef=new Hashtable<Integer,String>();
        int lsize=leaves.size();
        int nsize=nodes.size();
        String NetworkRoot=new String();

        if(Name.compareTo("")==0)
            return new Hashtable<Integer,String>();
        String CurrentRoot=Name;
        NetworkRoot=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+".";//models
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);
        NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+".";//pkg1
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);
        NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+".";//networks
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);        
        NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'));//myNetworkName
        String PkgNetwork=NetworkRoot+".";
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);  //dump myNetworkName   
        Network n=(Network)(this.lookupRef(NetworkRoot,(this.references.get("Network"))));
        NetworkRoot+=".";
        NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+"."; //put elements/probes
        CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);  //dump elements/probes           
       // NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+"."; //put elements/probes name
       // CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);  //dump elements/probes name                  
        int nbPoints=6;//better thna5 ?        
        while(!finished){
            boolean last=(CurrentRoot.lastIndexOf('.')==-1);
            if(last){
                for(int i=0;i<lsize;i++)
                {
                    Hashtable<String,Object> temp = getReferences().get(leaves.elementAt(i));
                    if (temp!=null) 
                    {
                        enum_=temp.keys();
                        while(enum_.hasMoreElements())
                        {
                            String k = (String)enum_.nextElement();
                            Object element = temp.get(k);
                           
                            if(k.startsWith(PkgNetwork)&& //in the network //n.getName()
                               k.endsWith("."+CurrentRoot))  //check if there is a point!!!
                            {
                                HashRef.put(new Integer(nbPoints), k);//NetworkRoot
                                return HashRef;
                            }
                        }
                    }
                }
            }
            String muche;
            if(last)
                muche="."+CurrentRoot;
            else
                muche="."+CurrentRoot.substring(0,CurrentRoot.indexOf('.'));
            boolean Found2=false;
            for(int i=0;i<nsize&&!Found2;i++)
            {
               Hashtable<String,Object> temp = getReferences().get(nodes.elementAt(i));
               if (temp!=null) 
               {
                   enum_=temp.keys();
                   while(enum_.hasMoreElements()&&!Found2)
                   {
                      String k = (String)enum_.nextElement();
                      Object element = temp.get(k);
                      if(k.startsWith(PkgNetwork)&& //in the network
                      k.endsWith(muche))  //check if there is a point!!!
                      {
                           ElementRef elref=(ElementRef)element;
                           n=(Network)(this.lookupRef(elref.elementClass.getTargetName(), (this.references.get("Network"))));
                           HashRef.put(new Integer(nbPoints), k);//NetworkRoot
                           
                           if(last) 
                               return HashRef;
                           else
                               Found2=true;
                      }
                   }
               }
            }
            if(CurrentRoot.indexOf('.')==-1)
                return HashRef;
            nbPoints++;
//            NetworkRoot+=CurrentRoot.substring(0,CurrentRoot.indexOf('.'))+"."; 
            CurrentRoot=CurrentRoot.substring(CurrentRoot.indexOf('.')+1);  
           // Vector v;
        }
        return null;
    }    
    
    public Object getPopRef(TreeRef myReference)
    {
        int MaxKey=-1,CurrentKey;
        Object o=null;
        Enumeration enum_;
        Hashtable<Integer,String> muche=getReferedTreeRef(myReference,myReference.getTargetName());
        enum_=muche.keys();	
	while(enum_.hasMoreElements()){
            Integer k=(Integer)enum_.nextElement();
            CurrentKey=k.intValue();
            if(CurrentKey>MaxKey){
                MaxKey=CurrentKey;
                o=muche.get(k);
            }
        }
        o=lookupRef((String)o,references.get("Population"));
        if(o instanceof Population)
            return o;
        else
            return null;
    }
    public Attachment lookAttachment(TreeRef myReference){
        Hashtable<String,Object> typeRefs=references.get("Attachment");
        if(typeRefs==null)
            return null;
	for(Enumeration enum_ = typeRefs.elements() ; enum_.hasMoreElements() ;) {
		Attachment aKey =(Attachment)enum_.nextElement();
		if(myReference==aKey.source)
                    return aKey;
	}
	
	return null;       
    }
    
    public Vector<Ref> getReferedHash(String str){
        return refered.get(str);
    }
    public Enumeration getReferences(String str){
        Hashtable<String,Object> muche=null;
        if(references!=null){
            muche=references.get(str);
            return muche.elements();
        }
        else
            return null;
        
    }
}

