package neuroml.util;


import neuroml.core.*;
 
import java.lang.reflect.*;


import java.io.*;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.File;
import java.net.URL;
import java.lang.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.*;
import java.util.jar.*;


/**
 * Static system class for neuroml.
 *
 * Contains wrappers for xmlin, xmlout and references.
 *
 * @version 1.0, 25 July 2001   
 *                 
 */
public class NeuromlSys {
    public static JarClassLoader classLoader=new JarClassLoader();//instanciation added by JAG
    public static String lastNamespaceName = "";

    /**
     * Load an XML file
     *
     * @param xmlfilename Name of the xml description file     
     * @return Top level object
     *                 
     */
    public static Object loadXML( String xmlfilename ){
	return XMLIn.loadXML(xmlfilename); 
    }

    /** Convert an XML string to an object
     */
    public static Object fromString( String xmltext ){
	return XMLIn.fromString(xmltext);
    }
    
    /** Write all fields 
     */
    public static String toString(Object top) {
	return XMLOut.toString(top);
    }
    
    /** Write all fields to a file */
    public static void toFile(Object top, String filename) {
	XMLOut.toFile( top, filename);
    }

    /** Write all fields, user specified top level tags
     */
    public static String toString(Object top, String topTag, String bottomTag) {
	return XMLOut.toString(top, topTag, bottomTag);
    }

    /** Write selected fields to XML
     */
    public static String toString(Field[] selectedFields, Object top) {
       return XMLOut.toString(selectedFields, top);
    }

    /** Deal with references / namespaces
     */
    static Hashtable<String,Namespace> namespaces = new Hashtable<String,Namespace>();

    /**
     * Make a new namespace (for references - one namespace / file)
     */
    public static Namespace makeNamespace(String name) { 
	Namespace n = new Namespace( name );
	lastNamespaceName = name;
	namespaces.put( name, n );
	return n;
    }

    /**
     * Returns namespace
     */
    public static Namespace getNamespace(String name) { 
	return namespaces.get(name);
    }

    /**
     * Returns namespace
     */
    public static Namespace getNewestNamespace() { 
	return namespaces.get(lastNamespaceName);
    }

    /**
     * Delete a namespace
     */
    public static void removeNamespace(String name) { 
	namespaces.remove( name );
    }
 
    /** Retrieve an Object from a Ref
     */
    public static Object deRef(Reference ref, String namespace ) {
	Namespace n = getNamespace(namespace);
	if (n != null) return n.deRef( ref ); else return null;
    }

    /** Add a reference in given namespace
     */
    public static void addRef(String key, Object obj, String namespace ) {
	Namespace n = getNamespace(namespace);
	if (n==null) n = makeNamespace(namespace);
	n.addRef( key, obj );
    }
    
    /** Add all references in the package
     */
    public static void addReferences(Object pkg, String namespace) {
	Namespace n = getNamespace(namespace);
	if (n==null) n = makeNamespace(namespace);
	n.addReferences( pkg );
    }

    /** Return the hashtable of all possible targets
     */
    public static Hashtable getNamespaces(){ 
	return namespaces;
    }

    /** Remove a reference in given namespace
     */
    public static void removeRef( String key, Object obj, String namespace) {
	Namespace n = getNamespace(namespace);
	if (n != null) n.removeRef( key, obj );
    }

    /** Try to make an object */
    private static Object createObject(String className) {
        Object object = null;
        try {
            Class classDefinition = Class.forName(className);
            object = classDefinition.newInstance();         
        } catch (Exception e) {
        } 
        return object;
    }

    /** Try to make a class 
     */
    public static Class createClass(String className) {
        Class classDefinition =null;
	try {
	    classDefinition = Class.forName(className); 
        }
        catch (Exception except) { 
        } 
        return classDefinition;
    }
  

    /** Try to create an object of the given class from a jar file 
     *  using the jarclassloader
     */
    public static Class getClassFromJar(String className) {
	Class classDefinition = null;
	if (classLoader == null)
	    classLoader = new JarClassLoader();	
	try {
	    classDefinition = classLoader.getLoadedClass(className);
	    if (classDefinition == null)
		classDefinition = classLoader.loadClassFromJar(className);
	} catch (java.lang.LinkageError je) {
	    try {		
		classDefinition = classLoader.getLoadedClass(className);		
	    } catch (Exception se){
		System.out.println("getClassFromJar subexception "+se);
	    }
	}
	return classDefinition;
    }

    /** Get full class given a sort neuroml class name (i.e. without 
     * the package prefix)
     */
    public static Class getClassFromShortName( String cname ) {
	String[] packageslist = neuroml.NeuromlPkg.getPackageList();
	String packageName = new String();
	Class c = createClass(cname);
	for (int j=0;j<packageslist.length && c==null;j++){
	    c = createClass(packageslist[j]+"."+cname);
	}
	return c;	
    }


    /** Make a object given the class name and from a jar file
     ** ?? Is this the best name for this method ??
     */
    public static Object makeObjectFromJar( String cname ) {
	Object object = null;
	Class c = getClassFromJar(cname);
	if (c!=null) {
	    try {
		object = c.newInstance();         
	    } catch (Exception e) {
	    } catch (java.lang.LinkageError je){
		try {
		    object =  classLoader.getLoadedClass(cname).newInstance();
		} catch (Exception se) {
		    System.out.println("makeObjectFromJar last try exception : "+se);
		}
	    }
	}
	return object;
    }

    /** Make a neuroml object given the class name 
     *
     * WARNING - this may not work for private classes defined in a user's package
     */
    public static Object makeNeuromlObject(String cname) {
	String[] packageslist = neuroml.NeuromlPkg.getPackageList();
	String packageName = new String();
	Object o = null;
	for (int j=0;j<packageslist.length && o==null;j++){
	    o = createObject(packageslist[j]+"."+cname);
	} 
	if (o==null) o=createObject(cname);
	return o;
    }

    /** Add class names from a jar file to the namespace
     */
    public static void registerClassesFromJar(URL jarname) {
	registerClassesFromJar(jarname, getNewestNamespace() );
    }
    public static void registerClassesFromJar(URL jarname, Namespace ns) {
	InputStream inf=null;
	JarInputStream jinf=null;
	try {
	    inf = jarname.openStream();
	} catch (IOException iex){
	    System.out.println("registerClassesFromJar URL get InputStream io Exception "+iex);
	    return;
	} catch (Exception ex){
	    System.out.println("registerClassesFromJar URL get InputStream Exception "+ex);
	    return;
	}
	try {
	    jinf = new JarInputStream(inf);
	} catch (IOException jio){
	    System.out.println("registerClassesFromJar JarInputStream IO Exception "+jio);
	    return;		
	}

	ZipEntry ze;
	try {
	    boolean go = true;
	    // list all classes
	    Vector<String> classList = new Vector<String>();
	    while(go){
		ze = jinf.getNextEntry();
		if (ze==null) go = false;
		else { 
		    // format class name (from package/subpackage/name.class to package.subpackage.name )
		    // add only class names, ignore if not
		    if (ze.getName().endsWith("class")) {
			// add to namespace
			// remove ".class"
			String zclassname = ze.getName().substring(0,ze.getName().length()-6);
			zclassname = zclassname.replace('/','.');
			classList.add(zclassname);
		    }
		}
	    }
	    String[] classArray = new String[classList.size()];
	    for (int i=0;i<classList.size();i++){
		classArray[i] = classList.get(i);
	    }
	    // add the jar file in the loader claspath
	    NeuromlSys.classLoader.add(jarname);
	    if (ns==null) return; 
	    ns.initialiseDerivedFrom(classArray);
	} catch (ZipException zz){
	    System.out.println("ZipEntry ZIP Exception "+zz);	    
	} catch (IOException io){
	    System.out.println("ZipEntry IO Exception "+io);	    
	}	
    }
}
