/*
 * BasicChannelCell.java
 *
 * Created on 19 August 2004, 16:47
 */

package neuroml.util.module;
import neuroml.model.ions_channel.*;
import neuroml.core.*;
import neuroml.sim.run.RunnableCellFactory;
import neuroml.sim.run.SpikeSender;
import neuroml.sim.state.PopulationState;
import neuroml.util.NeuromlSys;
import neuroml.util.module.BasicRunnableCell;
import neuroml.model.I_Clamp.*;

import neuroml.sim.run.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;

/**
 *
 * @author  Jean-Alain Grunchec
 */
public class BasicChannelCell extends BasicRunnableCell implements ChannelCell {
    protected double v;
    protected double dv;
    protected double t;
    protected double surface;
    protected boolean spiking;
    protected Channel Channels[];
    //_Function ExternalIntensity_;
    
    protected double minTrigger;
    protected double gap_time;
    protected double FormerInternalIntensity=0;
    protected boolean formerStable=false;
    /** Creates a new instance of BasicChannelCell */
    /** Return true if the cell is left in a stable stage, false otherwise */
    public boolean advance(){
         double dt=((BasicChannelCellFactory)factory).dt;
         if(!stable||!formerStable){
            stable=true;
            for(int i=0;i<Channels.length;i++){
                 boolean muche=Channels[i].advance(v, dt);
                 stable&=muche;
            }
         }
         stable&=ExternalIntensity.advance(dt);
         t += dt;
         return false;
    }
    
    public final void solveU(){
         double TotalIntenstityChannel=getInternalCurrent();
         if(stable&&formerStable)
             return;
         for(int i=0;i<Channels.length;i++){
            TotalIntenstityChannel-=Channels[i].getChannelCurrent(v)*surface;
         }
         dv=TotalIntenstityChannel*((BasicChannelCellFactory)factory).dt/(((BasicChannelCellFactory)factory).Capacity_Membrane*surface);//*surface
         stable=(dv<0.001*((BasicChannelCellFactory)factory).dt)&&(dv>-0.001*((BasicChannelCellFactory)factory).dt);
         v+=dv;     
         
    } 
    
    public void checkSpiking(){
        if(!spiking){
            if(dv>((BasicChannelCellFactory)factory).dv_trigger)
                spiking=true;
        }
        else if(dv<=0){
             if(v>minTrigger){
                 out.sendSpike(t, (short)1); //send spikes on port 1...
             }
             spiking=false;
        }
    }
    
    public BasicChannelCell(RunnableCellFactory runnablecellfactory, PopulationState populationstate, int id, int index, SpikeSender spikesender, double IniV)
    {
        super(runnablecellfactory, populationstate,id, index , spikesender);
        factory = (BasicChannelCellFactory)runnablecellfactory;
        v = IniV+0.065;//shifting
        t = 0.0D;    
        spiking = false ;
        Object vect[]=(Object[])(((BasicChannelCellFactory)factory).Channels.toArray());
        Channels=new Channel[vect.length];
        for(int i=0;i<vect.length;i++){
            Channels[i]=((Channel)(((ModuleChannel)(vect[i])).module.parameters)).Clone();
        }
  //      ExternalIntensity_=(I_Clamp)(ExternalIntensity.module.parameters);
  //      ExternalIntensity_.reset(this.index);
        surface=((BasicChannelCellFactory)runnablecellfactory).diameter*((BasicChannelCellFactory)runnablecellfactory).diameter*java.lang.Math.PI;
        minTrigger=((BasicChannelCellFactory)factory).minV_trigger+0.065;
        gap_time=((BasicChannelCellFactory)factory).dv_trigger*((BasicChannelCellFactory)factory).dt; 
    }

    public void reset()
    {
        v = 0.0D;
        t = 0.0D;
        spiking = true ;
    }
//depends on synapses...
    public void handleSpike(double d, int i)
    {
    }

    public void handle(Object obj, double d, int i)
    {
    }

    public final void advance(double d)
    {
        
        while(t < d) 
        {                
             solveU();
             if(!stable&&!formerStable) checkSpiking();
             advance();
             if(index==0){
                 if(stable){
                    if(!formerStable){    
                        System.out.println("Stable\tTime="+t);
                        
                    }
                 }
                 else{
                     if(formerStable){    
                        System.out.println("UnStable\tTime="+t);
                        
                    }
                 }
             }
             formerStable=stable;
        }
    }

    public final Object request(String s)
    {
        return null;
    }

    public final Object query(Query q) {
        if(q instanceof DrawQuery){
            JPanel jPanelCells=new JPanel(new java.awt.GridBagLayout());
            GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints();
            gridBagConstraints.fill=GridBagConstraints.HORIZONTAL;
            JLabel jLabelVoltage=new JLabel("Voltage = "+(v-0.065));
            jPanelCells.add(jLabelVoltage,gridBagConstraints);
            return jPanelCells;
        }
        else
            return q.query(this);
    }    
    
       //If the neuron has some synapses, there might some internal currents to take into account
    public double getInternalCurrent(){
        double InternalIntensity=ExternalIntensity.getValue(t); //ExternalIntensity_.getExternalIntensity(t,this.index);
        FormerInternalIntensity-=InternalIntensity;//arbitrary
        if(FormerInternalIntensity>1e-18||FormerInternalIntensity<-1e-18)
            stable=false;
        else
            stable=true;
        FormerInternalIntensity=InternalIntensity;
        return InternalIntensity;
    }
    
    
    public final Channel getChannel(int i){
        return Channels[i];
    }
    
    public double getVoltage(double t){
        return v; 
    }      
    public final double getExternalIntensity(double t){
        return ExternalIntensity.getValue(t);
    }    
    public final double getInternalIntensity(double t){
        double TotalIntenstityChannel=0;
         for(int i=0;i<Channels.length;i++){
            TotalIntenstityChannel-=Channels[i].getChannelCurrent(v);
         }        
        TotalIntenstityChannel*=surface;
        return TotalIntenstityChannel;//no channels
    }     
    public final double getTotalIntensity(double t){
         double TotalIntenstityChannel=getInternalCurrent();
         for(int i=0;i<Channels.length;i++){
            TotalIntenstityChannel-=Channels[i].getChannelCurrent(v)*surface;
         }
         return TotalIntenstityChannel;
    }    
}


