/**
 * Example implementation of a runnable cell
 */

package neuroml.util.module;

import neuroml.sim.run.*;
import neuroml.sim.state.*;
import java.util.Hashtable;
import neuroml.model.I_Clamp.*;
import neuroml.model.MeasurementTool.*;

 
/** A runnable cell
 */
public class BasicRunnableCell implements RunnableCell, RequestableState, Queryable,PositionState {
    neuroml.sim.state.Shape s;
    /** The interface we use for sending spikes
     */
    transient protected SpikeSender out;

    /** The factory which produced us - from this we can 
     * get access to the NeuroML model
     */
    transient protected RunnableCellFactory factory;

    /** The parent population *instance* 
     */
    transient protected PopulationState parent;

    /** The state of this cell
     */
    protected ElementState state;
    
    /** Our index in parent population
     */
    protected int index;

    /** Our global id
     */
    protected int id;

    /** Our position
     */
    protected double x,y,z;
    
    /**
     * If a cell is in a stable stage AND external currents are stable too you don't need to make computation, which is the interest of discrete events simulations.
     */
    protected boolean stable = false;
    
    //protected boolean Measured = false; //if a cell has to sent some measures
    
    protected _Function ExternalIntensity;
    /** There has to be a null constructor, as this class
     * will go in a loadable module
     */
    transient protected MeasurementCollectionCellLevel myTool=new MeasurementCollectionCellLevel();
    
    transient protected int myTool_Size=0;
    
    public BasicRunnableCell() { 
    } 

    public BasicRunnableCell( RunnableCellFactory f, PopulationState p, int id, int index, SpikeSender ss) { 
	factory = f;
	parent = p;
	this.index = index;
	out = ss;	
	this.id = id; // *** This should be a global id ***
    }  

    /** Get the model of this cell
     */
    public final Object getModel() {
	return factory.getModel();
    }

    /** Return current cell state 
     */
    public final ElementState getState() {
	return this;
    }

    /** --- Implementation of RequestableState ----
     * request handler, to be overwritten by derived classes
     * @Deprecated !!!!
     */
    public Object request(String r) {
	return new Integer(0);
    }

    /** This isn't *yet* a method of neuroml.sim.state.ElementState...
     */
    public final int getIndex() { 
	return index;
    }
 
    /** Set ID
     */
    public final void setID(int id) { this.id = id; }
    
    /** Get ID
     */
    public final int getID() { return id; }

    /** Set position
     */
    public final void setPos(double x, double y, double z) {
	this.x = x;
	this.y = y;
	this.z = z;
    }

    /** Get X position
     */
    public final double getXPos() { return x; }
    
    /** Get Y position
     */
    public final double getYPos() { return y; }
    
    /** Get Z position
     */
    public final double getZPos() { return z; }
  
    /** Handle a spike on a given input 
     */
    public void handleSpike(double t, int input ) {
	//	System.out.println("Spike received at time "+t+" on input "+input);
    }       

    /** Handle a generic object on a given input
     */
    public void handle(Object o, double t, int input) {
	//	System.out.println("Object "+o.getClass()+" received at time "+t+" on input "+input);
    }

    /** Advance time to t */
    public void advance(double t) {
	//	System.out.println("Told to advance to "+t);
    }

    /** reset to time 0 */
    public void reset() {
	//	System.out.println("Told to reset");
    }

    /** Deal with queries
     */
    public Object query(Query q) {
	return q.query(this);
    }

    /** Set additional connection parameters 
     */
    public void setInputConnectionParameters(int cid, DestMethod dm, ConnectionRequest cr) {
	// if dm instanceof DMWIthWeight w = dm.getWeight()
	// set synapse[cid] to be a new synapse(w)
	// port = dm.getDstPortID()
	// delay = dm.getDelay()
	// can also do: Object o = cr.getSrcData(); synapse[cid]=o;
    }
    
    public void draw() {
        s.draw();
    }
    
    public Shape getShape() {
        return s;
    }
    
    public void setShape(Shape s) {
        this.s=s;
    }
    
    public void setMeasured(MeasurementCollection MeasurementList){
        if(MeasurementList==null)
            return;
        //this.Measured=true;
        myTool.Add(MeasurementList);
        myTool_Size++;
    }
    
    public void setMeasured(int SrcId, MeasurementCollection MeasurementList){
        ((MeasurementHandler)out).setMeasured(SrcId,MeasurementList);
    }
    
    public void setIntensity(_Function f){
        ExternalIntensity=f;
    }
    public _Function getIntensity(){
        return ExternalIntensity;
    }
    public double getVoltage(double t){
        return 0.0d; 
    }
    public double getExternalIntensity(double t){
        return 0.0d;
    }    
    public double getInternalIntensity(double t){
        return 0.0d;
    }        
    public double getTotalIntensity(double t){
        return 0.0d;
    }        
    
}
