/**
 * Neuroml source method - connect to all
 */

package neuroml.util.module;

import neuroml.sim.run.*;
import neuroml.sim.state.*;

 
/** A source method which connects to all members of the target population.
 */
public class BasicSourceMethod implements SourceMethod, Cloneable {
     
    public short srcPort = (short)1;

    static private boolean debug = true;

    /** There has to be a null constructor, as this class
     * will go in a loadable module
     */
    public BasicSourceMethod() {	
    }

    /** Send requests out to all members of target population.
     * Should this connect to target element instead?
     * @param sender This interface for sending requests
     * @param sourceElement The source element
     * @param destPop The target population
     * @param dm The DestMethod to run on the receiving end
     */
    public void sendRequests(ConnectionRequestSender crs, ElementState srcState, PopulationState dstState, DestMethod dm, boolean BuildingConnection) {   

	// *** If necessary, we could check out aspects of the sourceElement state here.
	// *** and include it the ConnectionRequest.sourceData object
	// *** and use the sendRequest( ConnectionRequest cr ) variant	
	for (int i=0; i<dstState.getNumElements(); i++) {
	    crs.sendRequest(srcState.getID(), srcPort, dstState.getElementID(i), dm,BuildingConnection);
	}
    }  
    
    public void Draw(java.awt.Graphics2D g, double x, double y) {
    }
    
    public short getSrcId() {
        return srcPort;
    }
    
}
