package neuroml.util.schema;

import neuroml.core.*; 
import neuroml.util.*; 
import neuroml.util.axiope.*; 

/** 
 * NeuromlSchema version of a "Field"
 */
public class NeuromlField extends NamedObject implements ValueAccessor {
    /** The type of the field */
    public String type;
    /** (optional) elementType of the field (if it's a set) */
    public String elementType = null; // For set/ref elements...
    /** (optional) default value */
    public String defaultValue = null; // for base elements
    /** (optional) description */
    public String desc = null; 
    
    public NeuromlField() { super(); }
    public NeuromlField( String n, String t ) { name = n; type = t; }
    public NeuromlField( String n, String t, String et ) { 
	name = n; type = t; elementType = et; 
    }
    public NeuromlField deepCopy() {
	NeuromlField ret = new NeuromlField( name, type, elementType );
	ret.defaultValue = defaultValue;
	ret.desc = desc;
	return ret;
    }

    /** Index of this field
     */
    private int fidx;

    /** Set the index of this field.
     * Called by NeuromlClass
     */
    public void setIndex(int i) { fidx = i; }
    public int getIndex() { return fidx; }

    /** Return the declared field type
     */
    public String getType() {
	return type;
    }

    /** Return the element type
     */
    public String getElementType() {
	return elementType;
    }

    /** Extract a value from a neuroml object
     */
    public Object getValue(Object o) { 
	if (o instanceof NeuromlObject) {
	    NeuromlObject no = (NeuromlObject) o;
	    Object v = no.fieldValues.get( fidx );
	    if (v instanceof NeuromlString) {
		return ((NeuromlString)v).value; // All Strings -> java String
	    } else if (v instanceof NeuromlVector) {
		return ((NeuromlVector)v).getElements(); // All sets -> java List
	    } else {
		return v;
	    }
	}
	return null; 
    }

}

