package nmlplay;

import java.io.Serializable;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;
import java.util.MissingResourceException;
import neuroml.environment.schedule.*;

/** A Dialog for editing NEOSIM parameters.
 *
 *  @author Holger Bringmann (hcb@lambdabase.de)
 *
 */
public abstract class NEOSIMSettings implements Serializable
{
  
  
  /** Kernel parameters. Use 'seq' to run in one thread, single machine, 'thr [n]' to run
   *  in [n] threads on a single machine or 'rmi' to run on multiple machines. See the NEOSIM
   *  documentation for more detail.
   */
  static String[] kernelParms = {"seq" };
  /** Player runtime. No Units supported so far.*/
  //static double runtime = 50;
  /** Player stepping.*/
  //static double stepping = 1;
  public static Schedule mySchedule;
  /** Try to load settings from property file*/
  static {
      mySchedule=new Schedule();
    if (NMLPlay_2.resources_NML != null) {
      try {
          String stuff=NMLPlay_2.resources_NML.getString("runtime");
          mySchedule.runtime=Double.parseDouble(stuff);
      } catch (MissingResourceException mre) {System.out.println("runtime");};
      try {
          mySchedule.stepping=Double.parseDouble(NMLPlay_2.resources_NML.getString("stepping"));
      } catch (MissingResourceException mre) {System.out.println("stepping");};
      try {
          mySchedule.visudelay =Double.parseDouble(NMLPlay_2.resources_NML.getString("visudelay"));
      } catch (MissingResourceException mre) {System.out.println("visudelay");};
      try {
          StringTokenizer st = new StringTokenizer(NMLPlay_2.resources_NML.getString("kernelparms"), " ");
          kernelParms = new String[st.countTokens()];
          for (int i=0; i<kernelParms.length; i++) kernelParms[i] = st.nextToken();
      } catch (MissingResourceException mre) {System.out.println("parms");};
      
    }
    else{
        mySchedule.runtime=10;
        mySchedule.stepping=0.1;
        mySchedule.visudelay=0.1;
        mySchedule.command="seq";
    }
    mySchedule.ViewOnOff=true;
    mySchedule.DebugOnOff=false;
  }

  
  /** The GUI component to edit these settings.*/
  //volatile don't know why it is volatile
  static NEOSIMSettingsDialog dialog;
  
  /** Popups the NEOSIM settings dialog.
   *  @param frame the parent frame
   */
  static void configure(JFrame frame) {
    dialog = new NEOSIMSettingsDialog(frame);  // should be reusable --> dispose, hide ?!?!?
    dialog.loadContent();
    dialog.popup(frame);
  }
  
  private static class NEOSIMSettingsDialog extends JDialog implements ActionListener {
    
    JTextField tf1, tf2, tf3, tf4;
    JCheckBox tc1, tc2;
    
    NEOSIMSettingsDialog(JFrame frame) {
      super(frame, "NEOSIM Settings", true);
      
      JPanel root = (JPanel)getContentPane();
      JPanel inputPanel = new JPanel();
      
      root.setLayout(new BoxLayout(root, BoxLayout.Y_AXIS));
      inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.Y_AXIS));
      
      tf1 = new JTextField();
      tf2 = new JTextField();
      tf3 = new JTextField();
      tf4 = new JTextField();
      tc1 = new JCheckBox("Visualiz. On/Off");
      tc2 = new JCheckBox("Debug On/Off");
      loadContent();
      
      tf1.setBorder( BorderFactory.createTitledBorder("Kernel Parameters"));
      tf1.setColumns(20);
      inputPanel.add(tf1);
      tf2.setBorder( BorderFactory.createTitledBorder("Runtime"));
      inputPanel.add(tf2);
      tf3.setBorder( BorderFactory.createTitledBorder("Stepping"));
      inputPanel.add(tf3);
      tf4.setBorder( BorderFactory.createTitledBorder("Visualiz. Delay"));
      inputPanel.add(tf4);
     
      inputPanel.add(tc1);
      inputPanel.add(tc2);
      inputPanel.setBorder(BorderFactory.createTitledBorder(""));
      
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
      
      JButton cancel = new JButton("Cancel");
      buttonPanel.add(cancel);
      cancel.addActionListener(this);
      cancel.setActionCommand("cancel");
      
      JButton ok = new JButton("ok");
      buttonPanel.add(ok);
      ok.addActionListener(this);
      ok.setActionCommand("ok");
      
      root.add(inputPanel);
      root.add(buttonPanel);
      pack();
      setResizable(false);
    }
    
    void loadContent() {
     // StringBuffer sb = new StringBuffer();
     // for (int i=0; i<kernelParms.length; i++) sb.append(kernelParms[i]+" ");
     // tf1.setText(sb.toString());
      tf1.setText(mySchedule.command);
      tf2.setText(String.valueOf(mySchedule.runtime));
      tf3.setText(String.valueOf(mySchedule.stepping));
      tf4.setText(String.valueOf(mySchedule.visudelay));
      tc1.setSelected(mySchedule.ViewOnOff);
      tc2.setSelected(mySchedule.DebugOnOff);
    }

    void popup(JFrame frame) {
      setLocationRelativeTo(frame);
      Dimension d1 = frame.getSize();
      Point p = frame.getLocation();
      Dimension d2 = getSize();
      setLocation(p.x+ (d1.width-d2.width)/2, p.y+(d1.height-d2.height)/2);
      setVisible(true);
    }
    
    public void actionPerformed(ActionEvent e) {
      if (e.getActionCommand().equals("cancel")) this.dispose();
      if (e.getActionCommand().equals("ok")) {
        StringTokenizer st = new StringTokenizer(tf1.getText(), " ");
        kernelParms = new String[st.countTokens()];
        for (int i=0; i<kernelParms.length; i++) kernelParms[i] = st.nextToken();
        try {
          mySchedule.runtime=Double.parseDouble(tf2.getText());
          mySchedule.stepping=Double.parseDouble(tf3.getText());
          mySchedule.command=tf1.getText();
          mySchedule.visudelay=Double.parseDouble(tf4.getText());
          mySchedule.ViewOnOff=tc1.isSelected();
          mySchedule.DebugOnOff=tc2.isSelected();
          neosim.renderer.NetvisCtrl.setRefreshVisualization(mySchedule.visudelay,mySchedule.ViewOnOff);
          
          this.dispose();
        } catch (NumberFormatException nfe) {}
      }
    }
  }
}


